/*!
 * Copyright (C) 2010-2012 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * PVDaemonService
 * PVDaemon
 * 
 * @version 0.01
 * @author lyq (kazovision.liang@gmail.com)
 */

package com.kazovision.pvdaemon;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.lang.Thread.UncaughtExceptionHandler;
import java.util.Date;
import java.util.Locale;

import android.app.AlarmManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.net.wifi.WifiManager;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.widget.Toast;

import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.communicate.Communicate;
import com.kazovision.pvdaemon.download.Download;
import com.kazovision.pvdaemon.externalcontrol.ExternalControl;

public class PvdaemonService  extends Service {
	private static final String TAG = "PvdaemonService";
	private NotificationManager mNotificationManager;
	private static final Date mStartTime = new java.util.Date();
	
	static public Date GetStartTime() {
		return mStartTime;
	}
	
	public class PvdaemonServiceImpl extends IPvdaemonService.Stub
	{
		@Override
		public String GetDebugInfo() throws RemoteException {
			UserPath userpath = new UserPath();
			return String.format("UserPath: %s", userpath.GetUserPath());
		}
		
		@Override
		public void StartPlayer() throws RemoteException {
			Log.v(TAG, "Starting player.");
			Intent intent = new Intent("com.kazovision.pvplayer.Run");
			intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK); //仰ӣҪ
			startActivity(intent);
		}
	}
	
	@Override
	public void onCreate() {
		super.onCreate();
		//ʱԶ
		Thread.setDefaultUncaughtExceptionHandler(RestartHandler);
		
		Log.v(TAG, "Pvdaemon create.");
		Toast toast = Toast.makeText(this, String.format("Version: %s", Config.SYSTEM_BUILD_VERSION), Toast.LENGTH_LONG);
		toast.show();
		
		//ϵͳ֪ͨʾCopyrightϢ
		mNotificationManager = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
		Notification notification = new Notification(R.drawable.icon, "PVDAEMON", System.currentTimeMillis());
		notification.flags = Notification.FLAG_NO_CLEAR;
		PendingIntent contentIntent = PendingIntent.getActivity(this, 0, new Intent(""), 0);
		notification.setLatestEventInfo(this, "PVDAEMON", "Copyright(C) 2010-2013 Kazo Vision.", contentIntent);
		mNotificationManager.notify(0, notification);
		
		//ȡնMACַ
		String mac = ((WifiManager)getSystemService(Context.WIFI_SERVICE)).getConnectionInfo().getMacAddress();
		if (mac == null) {
			Log.e(TAG, "Error when get mac, use last mac instead.");
			mac = GetMacInfo();
		}
		if (mac == null) {
			Log.e(TAG, "Error when get mac, use 00:00:00:00:00:00 instead.");
			mac = "00:00:00:00:00:00";
		}
		mac = mac.toUpperCase(Locale.ENGLISH);
		if (! mac.equals(GetMacInfo())) {
			try {
				SetMacInfo(mac);
			}
			catch (Exception e) {
				Log.e(TAG, "Error when set mac.");
				notification.setLatestEventInfo(this, "PVDAEMON", "Error when set mac.", contentIntent);
				mNotificationManager.notify(0, notification);
			}
		}
		Download.Initialize(this);
		Communicate.Initialize(this);
		ExternalControl.Initialize(this);
	}
	
	public UncaughtExceptionHandler RestartHandler = new UncaughtExceptionHandler() {
		@Override
		public void uncaughtException(Thread thread, Throwable ex) {
			Intent intent = new Intent();
			intent.setClassName("com.kazovision.pvdaemon", "com.kazovision.pvdaemon.PvdaemonService");
			PendingIntent restartintent = PendingIntent.getActivity(getApplicationContext(), 0, intent, Intent.FLAG_ACTIVITY_NEW_TASK);
			AlarmManager alarmmanager = (AlarmManager) getSystemService(Context.ALARM_SERVICE);
			alarmmanager.set(AlarmManager.RTC, System.currentTimeMillis() + 1000, restartintent); // 1ӺӦ 
			android.os.Process.killProcess(android.os.Process.myPid());
		}
	};
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		//ϵͳڴ治㣬serviceϵͳkillϵͳԶؽservice
		return START_STICKY;
	}
	
	@Override
	public void onDestroy() {
		ExternalControl.Finalize();
		Communicate.Finalize();
		Download.Finalize();
		mNotificationManager.cancelAll();
		Log.v(TAG, "Pvdaemon destroy.");
		super.onDestroy();
	}
	
	@Override
	public IBinder onBind(Intent intent) {
		return new PvdaemonServiceImpl();
	}
	
	@Override
	public boolean onUnbind(Intent intent) {
		return super.onUnbind(intent);
	}
	
	private String GetMacInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "mac");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String mac = bufferedreader.readLine();
			bufferedreader.close();
			return mac;
		}
		catch (Exception e) {
			return null;
		}
	}
	
	private void SetMacInfo(String pMac) throws Exception {
		UserPath userpath = new UserPath();
		FileWriter filewriter = new FileWriter(userpath.GetUserPath() + "mac");
		BufferedWriter bufferedwriter = new BufferedWriter(filewriter);
		bufferedwriter.write(pMac);
		bufferedwriter.flush();
		bufferedwriter.close();
	}
}
