/*!
 * Copyright (C) 2010-2012 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * CommandCacheDB
 * ָ洢ݿࡣ
 * 
 * @version 0.01
 * @author lyq (kazovision.liang@gmail.com)
 */

package com.kazovision.pvdaemon.communicate;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandExecuteStatus;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandNotifyStatus;
import com.kazovision.pvdaemon.communicate.CommandDownloadInfo.FileDownloadStatus;

public class CommandCacheDB {
	private static final String TAG = "CommandCacheDB";
	
	private class CommandCacheDBHelper extends SQLiteOpenHelper {
		public CommandCacheDBHelper(Context pContext) {
			super(pContext, "commandcache", null, 1);
		}
		
		@Override
		public void onCreate(SQLiteDatabase pDatabase) {
			pDatabase.execSQL("CREATE TABLE 'commandcache' ("
							+ "'commandid' INTEGER,"
							+ "'updatetime' DATETIME,"
							+ "'contents' VARCHAR,"
							+ "'result' VARCHAR,"
							+ "'executestatus' INTEGER,"
							+ "'executeretrycount' INTEGER,"
							+ "'notifystatus' INTEGER,"
							+ "'downloadprogress' INTEGER)");
			pDatabase.execSQL("CREATE TABLE 'commanddownload' ("
							+ "'commandid' INTEGER,"
							+ "'fileurl' VARCHAR,"
							+ "'flag' INTEGER,"
							+ "'localfilename' VARCHAR,"
							+ "'downloadstatus' INTEGER,"
							+ "'downloadprogress' INTEGER)");
		}
		
		@Override
		public void onUpgrade(SQLiteDatabase pDatabase, int pOldVersion, int pNewVersion) {
			pDatabase.execSQL("DROP TABLE IF EXISTS 'commandcache')");
			pDatabase.execSQL("DROP TABLE IF EXISTS 'commanddownload')");
			onCreate(pDatabase);
		}
	};
	
	private static final int MAX_TEMPORARY_COMMAND_ID = 10;
	
	private List<CommandCacheInfo> mTemporaryCommandCacheInfoList;
	private SQLiteDatabase mDatabase;
	
	public CommandCacheDB(Context pContext) {
		mTemporaryCommandCacheInfoList = new ArrayList<CommandCacheInfo>();
		CommandCacheDBHelper helper = new CommandCacheDBHelper(pContext);
		mDatabase = helper.getWritableDatabase();
	};
	
	public List<CommandCacheInfo> GetCommandCacheInfoList() throws Exception {
		List<CommandCacheInfo> commandcacheinfolist = new ArrayList<CommandCacheInfo>();
		
		for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
			commandcacheinfolist.add(commandcacheinfo);
		}
		
		String sql = "SELECT commandid, updatetime, contents, result, executestatus, executeretrycount, notifystatus, downloadprogress FROM commandcache";
		String[] params = null;
		Cursor cursor = null;
		try {
			cursor = mDatabase.rawQuery(sql, params);
			while (cursor.moveToNext()) {
				CommandCacheInfo commandcacheinfo = new CommandCacheInfo();
				FillCommandCacheInfo(cursor, commandcacheinfo);
				commandcacheinfolist.add(commandcacheinfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return commandcacheinfolist;
	}
	
	public boolean IsCommandExists(int pCommandID) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
				if (commandcacheinfo.getCommandID() == pCommandID) {
					return true;
				}
			}
			return false;
		}
		else {
			String sql = "SELECT commandid FROM commandcache WHERE commandid = ?";
			String[] params = new String[]{String.valueOf(pCommandID)};
			Cursor cursor = null;
			try {
				cursor = mDatabase.rawQuery(sql, params);
				if (cursor.getCount() == 0){ //ָIDڡ
					return false;
				}
			}
			finally {
				if (cursor != null) {
					cursor.close();
				}
			}
			return true;
		}
	}
	
	public void AppendCommand(int pCommandID, String pContents) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			CommandCacheInfo commandcacheinfo = new CommandCacheInfo();
			commandcacheinfo.setCommandID(pCommandID);
			commandcacheinfo.setUpdateTime(new java.util.Date());
			commandcacheinfo.setContents(pContents);
			commandcacheinfo.setResult("");
			commandcacheinfo.setExecuteStatus(CommandExecuteStatus.WaitingForExecute);
			commandcacheinfo.setExecuteRetryCount(0);
			commandcacheinfo.setNotifyStatus(CommandNotifyStatus.Notified);
			commandcacheinfo.setDownloadProgress(0);
			mTemporaryCommandCacheInfoList.add(commandcacheinfo);
		}
		else {
			String sql = "INSERT INTO commandcache (commandid, updatetime, contents, result, executestatus, executeretrycount, notifystatus, downloadprogress) "
						+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
			String updatetime = df.format(new java.util.Date());
			Object[] params = new Object[]{pCommandID, updatetime, pContents, "", 
					CommandExecuteStatus.WaitingForExecute.ordinal(), 0, 
					CommandNotifyStatus.Notified.ordinal(), 0};
			mDatabase.execSQL(sql, params);
			//commanddownloadиָصϢ
			DeleteCommandDownload(pCommandID);
		}
	}
	
	public void ModifyCommand(int pCommandID, String pContents) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
				if (commandcacheinfo.getCommandID() == pCommandID) {
					commandcacheinfo.setUpdateTime(new java.util.Date());
					commandcacheinfo.setContents(pContents);
					commandcacheinfo.setResult("");
					commandcacheinfo.setExecuteStatus(CommandExecuteStatus.WaitingForExecute);
					commandcacheinfo.setExecuteRetryCount(0);
					commandcacheinfo.setNotifyStatus(CommandNotifyStatus.Notified);
					commandcacheinfo.setDownloadProgress(0);
					break;
				}
			}
		}
		else {
			String sql = "UPDATE commandcache SET updatetime = ?, contents = ?, result = ?, executestatus = ?, executeretrycount = ?, notifystatus = ?, downloadprogress = ? "
						+ "WHERE commandid = ?";
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
			String updatetime = df.format(new java.util.Date());
			Object[] params = new Object[]{updatetime, pContents, "", CommandExecuteStatus.WaitingForExecute.ordinal(),
					0, CommandNotifyStatus.Notified.ordinal(), 0, pCommandID};
			mDatabase.execSQL(sql, params);
			//commanddownloadиָصϢ
			DeleteCommandDownload(pCommandID);
		}
	}
	
	public void DeleteCommand(int pCommandID) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			for (int i=0; i<mTemporaryCommandCacheInfoList.size(); i++) {
				CommandCacheInfo commandcacheinfo = mTemporaryCommandCacheInfoList.get(i);
				if (commandcacheinfo.getCommandID() == pCommandID) {
					mTemporaryCommandCacheInfoList.remove(i);
					break;
				}
			}
		}
		else {
			String sql = "DELETE FROM commandcache WHERE commandid = ?";
			Object[] params = new Object[]{String.valueOf(pCommandID)};
			mDatabase.execSQL(sql, params);
			//commanddownloadиָصϢ
			DeleteCommandDownload(pCommandID);
		}
	}
	
	public List<CommandCacheInfo> PrepareNotify() throws Exception {
		List<CommandCacheInfo> commandcacheinfolist = new ArrayList<CommandCacheInfo>();
		
		for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
			if (commandcacheinfo.getNotifyStatus() == CommandNotifyStatus.NeedNotify || commandcacheinfo.getNotifyStatus() == CommandNotifyStatus.Notifying) {
				commandcacheinfo.setNotifyStatus(CommandNotifyStatus.Notifying);
				commandcacheinfolist.add(commandcacheinfo);
			}
		}
		
		//ȡ״̬Ϊ NeedNotify  Notifying ָϢ
		String sql = "SELECT commandid, updatetime, contents, result, executestatus, executeretrycount, notifystatus, downloadprogress "
					+ "FROM commandcache WHERE notifystatus = ? OR notifystatus = ?";
		String[] params = new String[]{Integer.toString(CommandNotifyStatus.NeedNotify.ordinal()),Integer.toString(CommandNotifyStatus.Notifying.ordinal())};
		Cursor cursor = null;
		try {
			cursor = mDatabase.rawQuery(sql, params);
			int commandcount = 0;
			while (cursor.moveToNext()) {
				++commandcount;
				if (commandcount > 10) {	//ÿ෵10ָΪصݹͨѶʱʧܡ
					break;
				}
				CommandCacheInfo commandcacheinfo = new CommandCacheInfo();
				FillCommandCacheInfo(cursor, commandcacheinfo);
				commandcacheinfolist.add(commandcacheinfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		
		//ѡָ״̬Ϊ Notifying
		sql = "UPDATE commandcache SET notifystatus = ? WHERE commandid = ?";
		for (CommandCacheInfo info : commandcacheinfolist) {
			Object[] object = new Object[]{CommandNotifyStatus.Notifying.ordinal(), info.getCommandID()};
			mDatabase.execSQL(sql, object);
		}
		
		return commandcacheinfolist;
	}
	
	public void FinishNotify(int pCommandID) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
				if (commandcacheinfo.getCommandID() == pCommandID && commandcacheinfo.getNotifyStatus() == CommandNotifyStatus.Notifying) {
					commandcacheinfo.setNotifyStatus(CommandNotifyStatus.Notified);
					break;
				}
			}
		}
		else {
			//ָ֪ͨ״̬Ϊ Notified 
			String sql = "UPDATE commandcache SET notifystatus = ? WHERE commandid = ? AND notifystatus = ?";
			Object[] params = new Object[]{CommandNotifyStatus.Notified.ordinal(), pCommandID, CommandNotifyStatus.Notifying.ordinal()};
			mDatabase.execSQL(sql, params);
			
			//жϸָǷִϲ֪ͨԱ㽫ɾ
			Cursor cursor = null;
			try {
				sql = "SELECT commandid FROM commandcache WHERE commandid = ? AND notifystatus = ? AND (executestatus = ? OR executestatus = ?)";
				String[] params2 = new String[]{Integer.toString(pCommandID), Integer.toString(CommandNotifyStatus.Notified.ordinal()), Integer.toString(CommandExecuteStatus.ExecuteSuccess.ordinal()), Integer.toString(CommandExecuteStatus.ExecuteFailed.ordinal())};
				cursor = mDatabase.rawQuery(sql, params2);
				if (cursor.moveToNext()) {
					sql = "DELETE FROM commandcache WHERE commandid = ?";
					params = new Object[]{pCommandID};
					mDatabase.execSQL(sql, params);
					sql = "DELETE FROM commanddownload WHERE commandid = ?";
					params = new Object[]{pCommandID};
					mDatabase.execSQL(sql, params);
				}
			}
			finally {
				if (cursor != null) {
					cursor.close();
				}
			}
		}
	}
	
	public void FinishAllNotifies() throws Exception {
		for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
			FinishNotify(commandcacheinfo.getCommandID());
		}
		
		Cursor cursor = null;
		try {
			String sql = "SELECT commandid FROM commandcache WHERE notifystatus = ?";
			String[] params = new String[]{Integer.toString(CommandNotifyStatus.Notifying.ordinal())};
			cursor = mDatabase.rawQuery(sql, params);
			while (cursor.moveToNext()) {
				int commandid = cursor.getInt(cursor.getColumnIndex("commandid"));
				FinishNotify(commandid);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
	}
	
	public void ResetExecuteStatus() throws Exception {
		for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
			if (commandcacheinfo.getExecuteStatus() == CommandExecuteStatus.Executing) {
				commandcacheinfo.setExecuteStatus(CommandExecuteStatus.WaitingForExecute);
			}
		}
		
		String sql = "UPDATE commandcache SET executestatus = ? WHERE executestatus = ?";
		Object[] params = new Object[]{CommandExecuteStatus.WaitingForExecute.ordinal(), CommandExecuteStatus.Executing.ordinal()};
		mDatabase.execSQL(sql, params);
	}
	
	/*!
	 * Ҫִеָδҵ򷵻null
	 */
	public CommandCacheInfo PrepareExecute() throws Exception {
		for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
			if (commandcacheinfo.getExecuteStatus() == CommandExecuteStatus.WaitingForExecute) {
				commandcacheinfo.setExecuteStatus(CommandExecuteStatus.Executing);
				return commandcacheinfo;
			}
		}
		
		CommandCacheInfo commandcacheinfo = null;
		Cursor cursor = null;
		//1.Ѱִ״̬Ϊ WaitingForExexute ָ
		try {
			String sql = "SELECT commandid, updatetime, contents, result, executestatus, executeretrycount, notifystatus, downloadprogress "
						+ "FROM commandcache WHERE executestatus = ?";
			String[] params = new String[]{Integer.toString(CommandExecuteStatus.WaitingForExecute.ordinal())};
			cursor = mDatabase.rawQuery(sql, params);
			if (cursor.moveToNext()) {
				commandcacheinfo = new CommandCacheInfo();
				FillCommandCacheInfo(cursor, commandcacheinfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		
		//2.Ѱִ״̬Ϊ WaitingForDownload  WaitingForDownloadExļȫ˵ָ
		if (commandcacheinfo == null){
			try {
				String sql = "SELECT commandid, updatetime, contents, result, executestatus, executeretrycount, notifystatus, downloadprogress "
							+ "FROM commandcache "
							+ "WHERE (commandid NOT IN (SELECT commandid FROM commanddownload WHERE downloadstatus = ?)) AND (executestatus = ? OR executestatus = ?)";
				String[] params = new String[]{Integer.toString(FileDownloadStatus.WaitingForDownload.ordinal()), Integer.toString(CommandExecuteStatus.WaitingForDownload.ordinal()), Integer.toString(CommandExecuteStatus.WaitingForDownloadEx.ordinal())};
				cursor = mDatabase.rawQuery(sql, params);
				if (cursor.moveToNext()) {
					commandcacheinfo = new CommandCacheInfo();
					FillCommandCacheInfo(cursor, commandcacheinfo);
				}
			}
			finally {
				if (cursor != null) {
					cursor.close();
				}
			}
		}
		
		//3.ָ ExecuteStatus Ϊ Executing ״̬
		if (commandcacheinfo != null){
			String sql = "UPDATE commandcache SET executestatus = ? WHERE commandid = ?";
			Object[] params = new Object[]{CommandExecuteStatus.Executing.ordinal(), commandcacheinfo.getCommandID()};
			mDatabase.execSQL(sql, params);
		}
		return commandcacheinfo;
	}
	
	public void FinishExecute(int pCommandID, CommandExecuteStatus pExecuteStatus, String pResult) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
				if (commandcacheinfo.getCommandID() == pCommandID && commandcacheinfo.getExecuteStatus() == CommandExecuteStatus.Executing) {
					commandcacheinfo.setResult(pResult);
					commandcacheinfo.setExecuteStatus(pExecuteStatus);
					commandcacheinfo.setNotifyStatus(CommandNotifyStatus.NeedNotify);
					break;
				}
			}
		}
		else {
			String sql = "UPDATE commandcache SET result = ?, executestatus = ?, notifystatus = ? "
					+ "WHERE commandid = ? AND executestatus = ?";
			Object[] params = new Object[]{pResult, pExecuteStatus.ordinal(), CommandNotifyStatus.NeedNotify.ordinal(), pCommandID, CommandExecuteStatus.Executing.ordinal()};
			mDatabase.execSQL(sql, params);
		}
	}
	
	public void ForceFinishExecute(int pCommandID, CommandExecuteStatus pExecuteStatus, String pResult) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
				if (commandcacheinfo.getCommandID() == pCommandID) {
					commandcacheinfo.setResult(pResult);
					commandcacheinfo.setExecuteStatus(pExecuteStatus);
					commandcacheinfo.setNotifyStatus(CommandNotifyStatus.NeedNotify);
					break;
				}
			}
		}
		else {
			String sql = "UPDATE commandcache SET result = ?, executestatus = ?, notifystatus = ? "
					+ "WHERE commandid = ?";
			Object[] params = new Object[]{pResult, pExecuteStatus.ordinal(), CommandNotifyStatus.NeedNotify.ordinal(), pCommandID};
			mDatabase.execSQL(sql, params);
		}
	}
	
	public int IncreaseExecuteRetryCount(int pCommandID) throws Exception {
		if (pCommandID <= MAX_TEMPORARY_COMMAND_ID) {
			for (CommandCacheInfo commandcacheinfo: mTemporaryCommandCacheInfoList) {
				if (commandcacheinfo.getCommandID() == pCommandID) {
					commandcacheinfo.setExecuteRetryCount(commandcacheinfo.getExecuteRetryCount() + 1);
					return commandcacheinfo.getExecuteRetryCount();
				}
			}
			return -1;
		}
		else {
			int executeretrycount = -1;
			Cursor cursor = null;
			try {
				String sql = "SELECT executeretrycount FROM commandcache WHERE commandid = ?";
				String[] params = new String[]{Integer.toString(pCommandID)};
				cursor = mDatabase.rawQuery(sql, params);
				if (cursor.moveToNext()) {
					executeretrycount = cursor.getInt(cursor.getColumnIndex("executeretrycount"));
					++ executeretrycount;
					sql = "UPDATE commandcache SET executeretrycount = ? WHERE commandid = ?";
					Object params2[] = new Object[]{executeretrycount, pCommandID};
					mDatabase.execSQL(sql, params2);
				}
			}
			finally {
				if (cursor != null) {
					cursor.close();
				}
			}
			return executeretrycount;
		}
	}
	
	public List<CommandDownloadInfo> GetCommandDownloadInfoList(int pCommandID) throws Exception {
		return GetCommandDownloadInfoList(pCommandID, 0);
	}
	
	public List<CommandDownloadInfo> GetCommandDownloadInfoList(int pCommandID, int pFlag) throws Exception {
		List<CommandDownloadInfo> commanddownloadinfolist = new ArrayList<CommandDownloadInfo>();
		Cursor cursor = null;
		try {
			String sql = "SELECT commandid, fileurl, flag, localfilename, downloadstatus, downloadprogress "
						+ "FROM commanddownload WHERE commandid = ? AND (flag = ? OR 0 = ?)";
			String[] params = new String[]{Integer.toString(pCommandID), Integer.toString(pFlag), Integer.toString(pFlag)};
			cursor = mDatabase.rawQuery(sql, params);
			while (cursor.moveToNext()) {
				CommandDownloadInfo commanddownloadinfo = new CommandDownloadInfo();
				FillCommandDownloadInfo(cursor, commanddownloadinfo);
				commanddownloadinfolist.add(commanddownloadinfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return commanddownloadinfolist;
	}
	
	public void AppendCommandDownload(int pCommandID, String pFileUrl) throws Exception {
		AppendCommandDownload(pCommandID,pFileUrl,0);
	}
	
	public void AppendCommandDownload(int pCommandID, String pFileUrl, int pFlag) throws Exception {
		String sql = "INSERT INTO commanddownload (commandid, fileurl, flag, localfilename, downloadstatus, downloadprogress) "
					+ "VALUES (?, ?, ?, ?, ?, ?)";
		Object[] params = new Object[]{pCommandID, pFileUrl, pFlag, "", FileDownloadStatus.WaitingForDownload.ordinal(), 0};
		mDatabase.execSQL(sql, params);
	}
	
	public void DeleteCommandDownload(int pCommandID) throws Exception {
		DeleteCommandDownload(pCommandID, 0);
	}
	
	public void DeleteCommandDownload(int pCommandID, int pFlag) throws Exception {
		String sql = "DELETE FROM commanddownload WHERE commandid = ? AND (flag = ? OR 0 = ?)";
		Object[] params = new Object[]{pCommandID, pFlag, pFlag};
		mDatabase.execSQL(sql, params);
	}
	
	public void UpdateFileDownloadProgress(String pFileUrl, int pDownloadProgress) throws Exception {
		//1.ļϢеĽȡ
		String sql = "UPDATE commanddownload SET downloadprogress = ? WHERE fileurl = ?";
		Object[] params = new Object[]{pDownloadProgress, pFileUrl};
		mDatabase.execSQL(sql, params);
		//2.ȡõǰļָcommandid
		Cursor cursor = null;
		try {
			sql = "SELECT commandid FROM commanddownload WHERE fileurl = ?";
			String[] params2 = new String[]{pFileUrl};
			cursor = mDatabase.rawQuery(sql, params2);
			//3.ָؽȡ
			boolean foundcommand = false;
			while (cursor.moveToNext()) {
				foundcommand = true;
				int commandid = cursor.getInt(cursor.getColumnIndex("commandid"));
				if (UpdateCommandDownloadProgress(commandid)) {
					//ָΪҪ֪ͨġ
					sql = "UPDATE commandcache SET notifystatus = ? WHERE commandid = ?";
					params = new Object[]{CommandNotifyStatus.NeedNotify.ordinal(), commandid};
					mDatabase.execSQL(sql, params);
				}
			}
			if (! foundcommand) {
				Log.e(TAG, String.format("Download command does not exists when update download progress, fileurl='%s'.", pFileUrl));
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
	}
	
	public void SetFileDownloadFinished(String pFileUrl, String pLocalFileName) throws Exception {
		//1.״̬ͽȡ
		String sql = "UPDATE commanddownload SET localfilename = ?, downloadstatus = ?, downloadprogress = ? "
					+ "WHERE fileurl = ?";
		Object[] params = new Object[]{pLocalFileName, FileDownloadStatus.Downloaded.ordinal(), 100, pFileUrl};
		mDatabase.execSQL(sql, params);
		//2.ȡõǰļָcommandid
		Cursor cursor = null;
		try {
			sql = "SELECT commandid FROM commanddownload WHERE fileurl = ?";
			String[] params2 = new String[] {pFileUrl};
			cursor = mDatabase.rawQuery(sql, params2);
			//3.ָؽȡ
			boolean foundcommand = false;
			while (cursor.moveToNext()) {
				foundcommand = true;
				int commandid = cursor.getInt(cursor.getColumnIndex("commandid"));
				if (UpdateCommandDownloadProgress(commandid)) {
					//ָΪҪ֪ͨġ
					sql = "UPDATE commandcache SET notifystatus = ? WHERE commandid = ?";
					params = new Object[]{CommandNotifyStatus.NeedNotify.ordinal(), commandid};
					mDatabase.execSQL(sql, params);
				}
			}
			if (! foundcommand) {
				Log.e(TAG, String.format("Download command does not exists when set file download finished, fileurl='%s'.", pFileUrl));
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
	}
	
	public boolean UpdateCommandDownloadProgress(int pCommandID) throws Exception {
		Cursor cursor = null;
		//1.ǰָеϢܵؽȡ
		int newprogress = 0;
		try {
			String sql = "SELECT downloadprogress FROM commanddownload WHERE commandid = ?";
			String[] params = new String[] {Integer.toString(pCommandID)};
			cursor = mDatabase.rawQuery(sql, params);
			int totalprogress = 0;
			int count = 0;
			while (cursor.moveToNext()) {
				int dprogress = cursor.getInt(cursor.getColumnIndex("downloadprogress"));
				totalprogress += dprogress;
				++ count;
			}
			newprogress = (count == 0) ? (0) : (totalprogress / count);
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		//2.ȡָǰؽȣжǷҪ¡
		int currentprogress = 0;
		try {
			String sql = "SELECT downloadprogress FROM commandcache WHERE commandid = ?";
			String[] params = new String[]{Integer.toString(pCommandID)};
			cursor = mDatabase.rawQuery(sql, params);
			if (cursor.moveToNext()) {
				currentprogress = cursor.getInt(cursor.getColumnIndex("downloadprogress"));
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		//3.ؽȷ˸ı䣬дָ
		if (newprogress != currentprogress) {
			String sql = "UPDATE commandcache SET downloadprogress = ? WHERE commandid = ?";
			Object[] params = new Object[]{newprogress, pCommandID};
			mDatabase.execSQL(sql, params);
			return true;
		}
		return false;
	}
	
	private void FillCommandCacheInfo(Cursor cur, CommandCacheInfo pCommandCacheInfo) {
		pCommandCacheInfo.setCommandID(cur.getInt(cur.getColumnIndex("commandid")));
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
		try {
			pCommandCacheInfo.setUpdateTime(df.parse(cur.getString(cur.getColumnIndex("updatetime"))));
		}
		catch (ParseException e) {
			Log.e(TAG, "Error when parse date time.");
		}
		pCommandCacheInfo.setContents(cur.getString(cur.getColumnIndex("contents")));
		pCommandCacheInfo.setResult(cur.getString(cur.getColumnIndex("result")));
		pCommandCacheInfo.setExecuteStatus(CommandExecuteStatus.values()[cur.getInt(cur.getColumnIndex("executestatus"))]);
		pCommandCacheInfo.setExecuteRetryCount(cur.getInt(cur.getColumnIndex("executeretrycount")));
		pCommandCacheInfo.setNotifyStatus(CommandNotifyStatus.values()[cur.getInt(cur.getColumnIndex("notifystatus"))]);
		pCommandCacheInfo.setDownloadProgress(cur.getInt(cur.getColumnIndex("downloadprogress")));
	}
	
	private void FillCommandDownloadInfo(Cursor cur, CommandDownloadInfo pCommandDownloadInfo) {
		pCommandDownloadInfo.setCommandID(cur.getInt(cur.getColumnIndex("commandid")));
		pCommandDownloadInfo.setFileUrl(cur.getString(cur.getColumnIndex("fileurl")));
		pCommandDownloadInfo.setFlag(cur.getInt(cur.getColumnIndex("flag")));
		pCommandDownloadInfo.setLocalFileName(cur.getString(cur.getColumnIndex("localfilename")));
		pCommandDownloadInfo.setDownloadStatus(FileDownloadStatus.values()[cur.getInt(cur.getColumnIndex("downloadstatus"))]);
		pCommandDownloadInfo.setDownloadProgress(cur.getInt(cur.getColumnIndex("downloadprogress")));
	}
}
