/*!
 * Copyright (C) 2010-2012 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * CommunicateController
 * ͨѶࡣ
 * 
 * @version 0.01
 * @author lyq (kazovision.liang@gmail.com)
 */

package com.kazovision.pvdaemon.communicate;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import android.content.Context;
import android.util.Log;
import android.widget.Toast;

import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandExecuteStatus;
import com.kazovision.pvdaemon.communicate.commandprocessor.BaseCommandProcessor;
import com.kazovision.pvdaemon.communicate.commandprocessor.DeleteContentCommandProcessor;
import com.kazovision.pvdaemon.communicate.commandprocessor.GetStatusCommandProcessor;
import com.kazovision.pvdaemon.communicate.commandprocessor.UnknownCommandProcessor;
import com.kazovision.pvdaemon.communicate.commandprocessor.UpdateContentCommandProcessor;
import com.kazovision.pvdaemon.communicate.commandprocessor.UpdateNotifyCommandProcessor;
import com.kazovision.pvdaemon.communicate.commandprocessor.UpdateScreenSizeCommandProcessor;
import com.kazovision.pvdaemon.communicate.commandprocessor.UpdateTerminalNameCommandProcessor;
import com.kazovision.pvdaemon.download.Download;
import com.kazovision.pvdaemon.download.DownloadController;
import com.kazovision.pvdaemon.download.DownloadHelper;

public class CommunicateController {
	private static final String TAG = "CommunicateController";
	private static final String PACKAGE_VERSION = "1.01";
	private static final int FILE_DOWNLOAD_RETRY_COUNT = 5;
	
	private Context mContext;
	private CommandCacheDB mCommandCacheDB;
	private DownloadController mDownloadController;
	private CommunicateThread mCommunicateThread;
	private CommandProcessThread mCommandProcessThread;
	
	static int CHECKSUM_TABLE_PVPLAYER[] = { 4727, 8178, 9854, 6781, 8804,
			1842, 3885, 2122, 4626, 2310, 1874, 4587, 991, 1487, 427, 136,
			9871, 7673, 675, 60, 3393, 7291, 1269, 7810, 7327, 9272, 4696,
			2475, 7853, 5706, 4902, 8748, 780, 4845, 6675, 5737, 4961, 5050,
			9771, 9452, 4120, 784, 3240, 6323, 3800, 4236, 7927, 7581, 2655,
			8677, 7621, 6857, 9817, 2132, 1806, 5837, 2537, 2476, 5497, 5312,
			826, 7534, 4128, 1481, 8325, 6288, 3952, 9831, 1116, 9779, 929,
			1465, 8672, 2799, 3571, 9180, 3712, 9971, 7709, 3196, 126, 2308,
			3653, 8817, 733, 2838, 8713, 923, 9064, 2403, 552, 2596, 1898,
			9211, 9332, 7748, 1762, 8249, 9137, 89 };

	public CommunicateController(Context pContext) {
		mContext = pContext;
		mCommandCacheDB = new CommandCacheDB(pContext);
		mCommunicateThread = new CommunicateThread(pContext, this);
		mCommandProcessThread = new CommandProcessThread(this);
		
		mDownloadController = Download.GetDownloadController();
		//עһDownloadHelperDownloadController֪ͨϢ
		DownloadHelper downloadhelper = new DownloadHelper(mCommandCacheDB);
		mDownloadController.RegisterDownloadHelper(downloadhelper);
		try {
			mCommandCacheDB.ResetExecuteStatus();
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when reset execute status '%s - %s'.", e.getClass().getName(), e.getMessage()));
		}
	}
	
	public void Start() {
		mCommunicateThread.start();
		mCommandProcessThread.start();
	}
	
	public void Stop() {
		mCommandProcessThread.interrupt();
		mCommunicateThread.interrupt();
	}
	
	/*!
	 * ׼Ҫ͵ָú߳̿ռִС
	 */
	public String PrepareRequestContents() throws Exception {
		DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
		DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
		Document document = documentBuilder.newDocument();
		/*
		 * <package>
		 */
		Element packageelement = document.createElement("package");
		document.appendChild(packageelement);
		packageelement.setAttribute("version", PACKAGE_VERSION);
		/*
		 * <package><terminal>
		 */
		Element terminalelement = document.createElement("terminal");
		packageelement.appendChild(terminalelement);
		terminalelement.setAttribute("mac", GetMacInfo());
		terminalelement.setAttribute("terminaltype", "MEDIAPLAYER");
		terminalelement.setAttribute("devicetype", "PVPLAYER");
		terminalelement.setAttribute("platform", "ANDROID");
		terminalelement.setAttribute("version", "0.01");
		/*
		 * <package><terminal><command>
		 */
		List<CommandCacheInfo> commandcacheinfolist = mCommandCacheDB.PrepareNotify();
		int checksum = CHECKSUM_TABLE_PVPLAYER[0];
		for (CommandCacheInfo info : commandcacheinfolist) {
			Document contentsdoc = null;
			try {
				StringReader stringReader = new StringReader(info.getContents());
				InputSource inputSource = new InputSource(stringReader);
				DocumentBuilderFactory documentBuildFactory = DocumentBuilderFactory.newInstance();
				documentBuilder = documentBuildFactory.newDocumentBuilder();
				contentsdoc = documentBuilder.parse(inputSource);
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when parse xml: '%s'", info.getContents()));
				throw new Exception("Error when parse xml.");
			}
			Element contents = contentsdoc.getDocumentElement();
			Element commandelement = document.createElement("command");
			terminalelement.appendChild(commandelement);
			commandelement.setAttribute("type", contents.getAttribute("type"));
			commandelement.setAttribute("commandid", contents.getAttribute("commandid"));
			commandelement.setAttribute("code", contents.getAttribute("code"));
			checksum += CHECKSUM_TABLE_PVPLAYER[Integer.parseInt(contents.getAttribute("code").toString())];
			if (info.getExecuteStatus() == CommandExecuteStatus.ExecuteSuccess) {
				commandelement.setAttribute("executestatus", "SUCCESS");
				if (info.getResult().length() > 0) {
					String result = "<result>" + info.getResult() + "</result>";
					Document resultdoc = null;
					try {
						StringReader stringReader = new StringReader(result);
						InputSource inputSource = new InputSource(stringReader);
						DocumentBuilderFactory documentBuildFactory = DocumentBuilderFactory.newInstance();
						documentBuilder = documentBuildFactory.newDocumentBuilder();
						resultdoc = documentBuilder.parse(inputSource);
					} catch (Exception e) {
						Log.e(TAG, String.format("Error when convert result to xml '%s'.", result));
						mCommandCacheDB.FinishNotify(info.getCommandID());
						continue;
					}
					Element resultelement = resultdoc.getDocumentElement();
					NodeList resultList = resultelement.getChildNodes();
					for (int i = 0; i < resultList.getLength(); i++) {
						Node resultNode = resultList.item(i);
						Node newResultNode = document.adoptNode(resultNode);
						commandelement.appendChild(newResultNode);
					}
				}
			}
			else if (info.getExecuteStatus() == CommandExecuteStatus.ExecuteFailed) {
				commandelement.setAttribute("executestatus", "FAILED");
				if (info.getResult().length() > 0) {
					String result = "<result>" + info.getResult() + "</result>";
					Document resultdoc = null;
					try {
						StringReader stringReader = new StringReader(result);
						InputSource inputSource = new InputSource(stringReader);
						DocumentBuilderFactory documentBuildFactory = DocumentBuilderFactory.newInstance();
						documentBuilder = documentBuildFactory.newDocumentBuilder();
						resultdoc = documentBuilder.parse(inputSource);
					} catch (Exception e) {
						Log.e(TAG, String.format("Error when convert result to xml '%s'.",result));
						mCommandCacheDB.FinishNotify(info.getCommandID());
						continue;
					}
					Element resultelement = resultdoc.getDocumentElement();
					NodeList resultList = resultelement.getChildNodes();
					for (int i = 0; i < resultList.getLength(); i++) {
						Node resultNode = resultList.item(i);
						Node newResultNode = document.adoptNode(resultNode);
						commandelement.appendChild(newResultNode);
					}
				}
			} else {
				commandelement.setAttribute("executestatus", "");
			}
		}
		packageelement.setAttribute("checksum", Integer.toString(checksum));
		
		TransformerFactory transFactory = TransformerFactory.newInstance();
		Transformer transformer = transFactory.newTransformer();
		StringWriter buffer = new StringWriter();
		transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		transformer.transform(new DOMSource(document), new StreamResult(buffer));
		String contenttext = buffer.toString();
		Log.v(TAG, String.format("Request contents=%s.", contenttext));
		return contenttext;
	}
	
	/*!
	 * յͨѶú߳̿ռִС
	 */
	boolean ProcessReturnContents(String pContents) {
		Log.v(TAG, String.format("Return contents=%s.", pContents));
		if (pContents.equals("ERROR")) {
			Log.e(TAG, "Server fatal error when process request contents. Force finish all notifies");
			try {
				mCommandCacheDB.FinishAllNotifies();
			}
			catch (Exception e) {
				Log.e(TAG, "Error when finish all notifies.");
			}
			return false;
		} else if (pContents.equals("UPGRADE")) {
			Log.e(TAG, "Terminal need to be upgrade.");
			Toast toast = Toast.makeText(mContext, "Please upgrade your software.", Toast.LENGTH_SHORT);
			toast.show();
			return false;
		}
		try {
			StringReader stringReader = new StringReader(pContents);
			InputSource inputSource = new InputSource(stringReader);
			DocumentBuilderFactory documentBuildFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder documentBuilder = documentBuildFactory.newDocumentBuilder();
			Document document = documentBuilder.parse(inputSource);
			Element packageelement = document.getDocumentElement();
			NodeList terminalNodeList = packageelement.getElementsByTagName("terminal");
			if (terminalNodeList.getLength() != 1) {
				Log.e(TAG, "Wrong number of terminal node.");
				return false;
			}
			Element terminalelement = (Element)terminalNodeList.item(0);
			String mac = terminalelement.getAttribute("mac");
			if (! mac.equals(GetMacInfo())) {
				Log.e(TAG, "Terminal's mac address not correct.");
				return false;
			}
			// ͨѶУ顣
			int checksum = CHECKSUM_TABLE_PVPLAYER[0];
			NodeList commandNodeList = terminalelement.getElementsByTagName("command");
			for (int i=0; i<commandNodeList.getLength(); i++) {
				Element commandelement = (Element)commandNodeList.item(i);
				checksum += CHECKSUM_TABLE_PVPLAYER[Integer.parseInt(commandelement.getAttribute("code"))];
			}
			if (checksum != Integer.parseInt(packageelement.getAttribute("checksum"))) {
				Log.e(TAG, "Package checksum not correct.");
				return false;
			}
			//ָ
			for (int i=0; i<commandNodeList.getLength(); i++) {
				Element commandelement = (Element)commandNodeList.item(i);
				String commandtype = commandelement.getAttribute("type");
				int commandid = Integer.parseInt(commandelement.getAttribute("commandid"));
				
				if (commandelement.hasAttribute("result")) {
					mCommandCacheDB.FinishNotify(commandid);
					String result = commandelement.getAttribute("result");
					if (result.equals("SUCCESS")) {
					}
					else if (result.equals("FAILED")) {
						Log.e(TAG, String.format("Server error: command failed. type: '%s'.", commandtype));
					}
					else if (result.equals("NOT_EXISTS")) {
						Log.e(TAG, String.format("Server error: command not exists. type: '%s'.", commandtype));
					}
					else if (result.equals("EXPIRATION")) {
						Log.e(TAG, String.format("Server error: command expiration. type: '%s'.", commandtype));
					}
					else if (result.equals("UNKNOWN")) {
						Log.e(TAG, String.format("Server error: command unknown error. type: '%s'.", commandtype));
					}
					else {
						Log.e(TAG, String.format("Server error: unknown result %s. type: '%s'.", result, commandtype));
					}
				}
				else {
					TransformerFactory transFactory = TransformerFactory.newInstance();
					Transformer transformer = transFactory.newTransformer();
					StringWriter buffer = new StringWriter();
					transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
					transformer.transform(new DOMSource(commandelement), new StreamResult(buffer));
					String contenttext = buffer.toString();
					if (mCommandCacheDB.IsCommandExists(commandid)) {
						mCommandCacheDB.ModifyCommand(commandid, contenttext);
					}
					else {
						mCommandCacheDB.AppendCommand(commandid, contenttext);
					}
				}
			}
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when process return contents: '%s - %s'.", e.getClass().getName(), e.getMessage()));
			return false;
		}
		return true;
	}
	
	public void ProcessOneCommand() throws Exception {
		CommandCacheInfo info = mCommandCacheDB.PrepareExecute();
		if (info == null) {		//ûҪִеָߺʱ
			return;
		}
		
		//TODO: ....ָǷڡ
		/*
		if (QDateTime::currentDateTime().addSecs(COMMAND_TIMEOUT_TIME * -3600) >= info.UpdateTime) {		//ָһʱڶûбִϣǿƽָ
			logger()->warn("Timeout when execute command. updatetime='%1'.", info.UpdateTime.toString("yyyy-MM-dd HH:mm:ss.z"));
			executestatus = pvdmCommandCacheInfo::ExecuteFailed;
			returncontents = "<error message=\"Timeout when execute command.\"/>";
		}
		else {		//ִָ
			ProcessCommand(info.Contents, executestatus, returncontents, wantretry);
		}
		*/
		Object[] result = ProcessCommand(info.getContents(), info.getExecuteStatus());
		CommandExecuteStatus executestatus = (CommandExecuteStatus)result[0];
		String returncontents = (String)result[1];
		boolean wantretry = (Boolean)result[2];
		
		//ָдݿС
		if (executestatus == CommandExecuteStatus.ExecuteFailed && wantretry) {	//ϣԸָ
			if (mCommandCacheDB.IncreaseExecuteRetryCount(info.getCommandID()) < FILE_DOWNLOAD_RETRY_COUNT) {
				executestatus = CommandExecuteStatus.WaitingForDownload;	//øָ´ִС
				returncontents = "";
			}
			else {
				Log.e(TAG, "Over max execute retry count, command failed.");
			}
		}
		mCommandCacheDB.FinishExecute(info.getCommandID(), executestatus, returncontents);
	}
	
	private Object[] ProcessCommand(String pCommandContents, CommandExecuteStatus pExecuteStatus) {
		Object[] result = new Object[3];
		result[0] = CommandExecuteStatus.ExecuteFailed;
		result[1] = "";
		result[2] = false;
		
		Document document = null;
		try {
			StringReader stringReader = new StringReader(pCommandContents);
			InputSource inputSource = new InputSource(stringReader);
			DocumentBuilderFactory documentBuildFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder documentBuilder = documentBuildFactory.newDocumentBuilder();
			document = documentBuilder.parse(inputSource);
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when parse xml: '%s - %s'", e.getClass().getName(), e.getMessage()));
			result[0] = CommandExecuteStatus.ExecuteFailed;
			result[1] = "<error message=\"Error when parse xml.\"/>";
			return result;
		}
		Element commandelement = document.getDocumentElement();
		
		String commandtype = commandelement.getAttribute("type");
		
		BaseCommandProcessor processor = null;
		/*!
		ȡͳơ
		*/
		if (commandtype.equals("GET_STATISTICS")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			ȡͼƬ
		*/
		else if (commandtype.equals("GET_SNAPSHOT")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			նơ
		*/
		else if (commandtype.equals("UPDATE_TERMINAL_NAME")) {
			processor = new UpdateTerminalNameCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			նʾߴ硣
		*/
		else if (commandtype.equals("UPDATE_SCREEN_SIZE")) {
			processor = new UpdateScreenSizeCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			½Ŀ
		*/
		else if (commandtype.equals("UPDATE_CONTENT")) {
			processor = new UpdateContentCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			ɾĿ
		*/
		else if (commandtype.equals("DELETE_CONTENT")) {
			processor = new DeleteContentCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			֪ͨ֡
		*/
		else if (commandtype.equals("UPDATE_NOTIFY")) {
			processor = new UpdateNotifyCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			ɾ֪ͨ֡
		*/
		else if (commandtype.equals("DELETE_NOTIFY")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			ģʽ
		*/
		else if (commandtype.equals("UPDATE_RUNNING_MODE")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			á
		*/
		else if (commandtype.equals("UPDATE_CONFIGURE")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			ѯն״̬
		*/
		else if (commandtype.equals("GET_STATUS")) {
			processor = new GetStatusCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			ѯش¼
		*/
		else if (commandtype.equals("GET_ERROR_LOG")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			
		*/
		else if (commandtype.equals("UPGRADE")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			رϵͳ
		*/
		else if (commandtype.equals("SHUTDOWN")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			ϵͳ
		*/
		else if (commandtype.equals("REBOOT")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			
		*/
		else if (commandtype.equals("RESET")) {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		/*!
			δָ֪
		*/
		else {
			processor = new UnknownCommandProcessor(mContext, mCommandCacheDB, mDownloadController);
		}
		
		processor.ProcessCommand(commandelement,pExecuteStatus);
		result[0] = processor.GetExecuteStatus();
		result[1] = processor.GetReturnContents();
		result[2] = processor.GetWantRetry();
		return result;
	}
	
	private String GetMacInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "mac");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String mac = bufferedreader.readLine();
			bufferedreader.close();
			if (mac == null) {
				mac = "";
			}
			return mac;
		}
		catch (Exception e) {
			return "";
		}
	}
}
