/*!
 * Copyright (C) 2010-2012 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * BaseCommandProcessor
 * ָࡣ
 * 
 * @version 0.01
 * @author tony (tonixinot@gmail.com)
 */

package com.kazovision.pvdaemon.communicate.commandprocessor;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;

import android.content.Context;
import android.util.Log;

import com.kazovision.pvdaemon.communicate.CommandCacheDB;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandExecuteStatus;
import com.kazovision.pvdaemon.communicate.CommandDownloadInfo;
import com.kazovision.pvdaemon.communicate.CommandDownloadInfo.FileDownloadStatus;
import com.kazovision.pvdaemon.download.DownloadController;

public abstract class BaseCommandProcessor {
	private static final String TAG = "BaseCommandProcessor";
	protected Context mContext;
	protected CommandCacheDB mCommandCacheDB;
	protected DownloadController mDownloadController;
	protected CommandExecuteStatus mExecuteStatus;
	protected String mReturnContents;
	protected boolean mWantRetry;
	
	public BaseCommandProcessor(Context pContext, CommandCacheDB pCommandCacheDB, DownloadController pDownloadController) {
		mContext = pContext;
		mCommandCacheDB = pCommandCacheDB;
		mDownloadController = pDownloadController;
		mExecuteStatus = CommandExecuteStatus.ExecuteFailed;
		mReturnContents = "";
		mWantRetry = false;
	}
	
	public abstract void ProcessCommand(Element pCommandElement, CommandExecuteStatus pExecuteStatus);
	
	public CommandExecuteStatus GetExecuteStatus() {
		return mExecuteStatus;
	}
	
	public String GetReturnContents() {
		return mReturnContents;
	}
	
	public boolean GetWantRetry() {
		return mWantRetry;
	}
	
	/*!
	 * XMLָϢеַת
	 */
	public String EscapeXmlString(String pString) {
		String result = pString;
		result = result.replaceAll("<", "&lt;");
		result = result.replaceAll(">", "&gt;");
		result = result.replaceAll("&", "&amp;");
		result = result.replaceAll("'", "&apos;");
		result = result.replaceAll("\"", "&quot;");
		return result;
	}

	public String FormatErrorMessage(String pErrorMessage) {
		String xml = "<error message=\"" + EscapeXmlString(pErrorMessage) + "\"/>";
		return xml;
	}
	
	/*!
		Ҫصļб pvdmDownload ģ飬ͬʱڱݿм¼ָӦļϢ
		@param pCommandID ļIDš
		@param pFileUrlList ҪصļURLб
		@return ʧʱfalse
	 */
	protected void DownloadFiles(int pCommandID, List<String[]> pFileUrlList, int pFlag) throws Exception {
		if (pFileUrlList.size() == 0) {
			Log.e(TAG, "The file url list is empty.");
			throw new Exception("The file url list is empty.");
		}
		mCommandCacheDB.DeleteCommandDownload(pCommandID, pFlag);
		for (String[] fileurl : pFileUrlList){
			mDownloadController.DownloadFile("", fileurl[0], fileurl[1]);
			mCommandCacheDB.AppendCommandDownload(pCommandID, fileurl[0], pFlag);
		}
	}
	
	/*!
		Ҫصļ pvdmDownload ģ飬ͬʱڱݿм¼ָӦļϢ
		@param pCommandID ļָIDš
		@param pFileUrl ҪصļURLַ
		@return ʧʱfalse
	 */
	protected void DownloadFiles(int pCommandID, String[] pFileUrl, int pFlag) throws Exception {
		List<String[]> fileurllist = new ArrayList<String[]>();
		fileurllist.add(pFileUrl);
		DownloadFiles(pCommandID, fileurllist, pFlag);
	}
	
	protected List<String[]> GetDownloadedFilesList(int pCommandID, int pFlag) throws Exception {
		List<String[]> fileinfolist = new ArrayList<String[]>();
		List<CommandDownloadInfo> commanddownloadinfolist;
		commanddownloadinfolist = mCommandCacheDB.GetCommandDownloadInfoList(pCommandID, pFlag);
		for (CommandDownloadInfo commanddownloadinfo : commanddownloadinfolist){
			if (commanddownloadinfo.getDownloadStatus() != FileDownloadStatus.Downloaded) {
				Log.e(TAG, String.format("Have unfinished download info '%s'.", commanddownloadinfo.getFileUrl()));
				throw new Exception(String.format("Have unfinished download info '%s'.", commanddownloadinfo.getFileUrl()));
			}
			String[] fileinfo = new String[2];
			fileinfo[0] = commanddownloadinfo.getFileUrl();
			fileinfo[1] = commanddownloadinfo.getLocalFileName();
			fileinfolist.add(fileinfo);
		}
		return fileinfolist;
	}
	
	protected String[] GetDownloadedFiles(int pCommandID, int pFlag) throws Exception {
		List<String[]> fileinfolist = GetDownloadedFilesList(pCommandID, pFlag);
		if (fileinfolist.size() != 1){
			Log.e(TAG, "Downloaded file number not correct.");
			throw new Exception("Downloaded file number not correct.");
		}
		return fileinfolist.get(0);
	}
}