/*!
 * Copyright (C) 2010-2013 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * DeleteContentCommandProcessor
 * ɾĿָࡣ
 * 
 * @version 0.01
 * @author tony (tonixinot@gmail.com)
 */

package com.kazovision.pvdaemon.communicate.commandprocessor;

import java.io.File;
import java.io.FilenameFilter;
import java.io.StringReader;
import java.util.List;
import java.util.Locale;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import android.content.Context;
import android.util.Log;

import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.communicate.CommandCacheDB;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandExecuteStatus;
import com.kazovision.pvdaemon.download.DownloadController;
import com.kazovision.pvdaemon.playercontrol.PlayerController;

public class DeleteContentCommandProcessor extends BaseCommandProcessor {
	private static final String TAG = "DeleteContentCommandProcessor";
	
	public DeleteContentCommandProcessor(Context pContext, CommandCacheDB pCommandCacheDB, DownloadController pDownloadController) {
		super(pContext, pCommandCacheDB, pDownloadController);
	}

	@Override
	public void ProcessCommand(Element pCommandElement,CommandExecuteStatus pExecuteStatus) {
		try {
			NodeList informationNodeList = pCommandElement.getElementsByTagName("information");
			Element informationelement = (Element) informationNodeList.item(0);
			String contenttype = informationelement.getAttribute("contenttype");
			if (contenttype == null || contenttype.equals("")) {
				contenttype = "NORMAL";
			}
			String informationid = informationelement.getAttribute("id");
			
			UserPath userpath = new UserPath();
			if (contenttype.equals("NORMAL") || contenttype.equals("ONLINE")) {
				String contentspath = userpath.GetUserSubPath("contents");
				if (informationid.equals("*")) {
					Log.i(TAG, "Delete all contents.");
					DeleteAllContentFiles(contentspath);
				}
				else {
					Log.i(TAG, String.format("Delete content '%s'.", informationid));
					DeleteContentFile(contentspath, informationid);
				}
			}
			else if (contenttype.equals("TIMEDPLAY")) {
				String contentspath = userpath.GetUserSubPath("contents_timedplay");
				if (informationid.equals("*")) {
					Log.i(TAG, "Delete all contents.");
					DeleteAllContentFiles(contentspath);
				}
				else {
					Log.i(TAG, String.format("Delete content '%s'.", informationid));
					DeleteContentFile(contentspath, informationid);
				}
			}
			else {
				Log.e(TAG, String.format("Unknown content type '%s'.", contenttype));
				throw new Exception("Unknown content type.");
			}
			
			//֪ͨ
			PlayerController playercontroller = new PlayerController(mContext);
			playercontroller.ReloadContent(contenttype);
			
			//пҪɾĽĿָļأδִɣ˱ָбҵӦĽĿָȡ
			CancelUpdateContentCommand(informationid);
			
			mExecuteStatus = CommandExecuteStatus.ExecuteSuccess;
			mReturnContents = "";
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when process command '%s - %s'.", e.getClass().getName(), e.getMessage()));
			mExecuteStatus = CommandExecuteStatus.ExecuteFailed;
			mReturnContents = FormatErrorMessage(e.getClass().getName() + "-" + e.getMessage());
		}
	}
	
	private void DeleteAllContentFiles(String pContentPath) throws Exception {
		File dir = new File(pContentPath);
		FilenameFilter filter = new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return name.toLowerCase(Locale.ENGLISH).endsWith(".script");
			}
		};
		File[] files = dir.listFiles(filter);
		if (files == null) {
			return;
		}
		for (int i = 0; i < files.length; i++) {
			File file = files[i];
			if (! file.delete()) {
				throw new Exception(String.format("Error when delete script file '%s'.", file.getName()));
			}
			int index = file.getName().lastIndexOf(".");
			File infofile = new File(pContentPath + "/" + file.getName().substring(0, index) + ".info");
			if (infofile.exists()) {
				if (! infofile.delete()) {
					throw new Exception(String.format("Error when delete info file '%s'.", infofile.getName()));
				}
			}
		}
	}
	
	private void DeleteContentFile(String pContentPath, String pInformationID) throws Exception {
		File scriptfile = new File(pContentPath + "/" + pInformationID + ".script");
		if (scriptfile.exists()) {
			if (! scriptfile.delete()) {
				throw new Exception(String.format("Error when delete script file '%s'.", scriptfile.getName()));
			}
		}
		
		File infofile = new File(pContentPath + "/" + pInformationID + ".info");
		if (infofile.exists()) {
			if (! infofile.delete()) {
				throw new Exception(String.format("Error when delete info file '%s'.", infofile.getName()));
			}
		}
	}
	
	private void CancelUpdateContentCommand(String pInformationID) throws Exception {
		List<CommandCacheInfo> commandcacheinfolist = mCommandCacheDB.GetCommandCacheInfoList();
		for (CommandCacheInfo commandcacheinfo: commandcacheinfolist) {
			if (commandcacheinfo.getExecuteStatus() == CommandExecuteStatus.ExecuteSuccess
					|| commandcacheinfo.getExecuteStatus() == CommandExecuteStatus.ExecuteFailed) {
				//Ѿִɵָ
				continue;
			}
			Document document = null;
			try {
				StringReader stringReader = new StringReader(commandcacheinfo.getContents());
				InputSource inputSource = new InputSource(stringReader);
				DocumentBuilderFactory documentBuildFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder documentBuilder = documentBuildFactory.newDocumentBuilder();
				document = documentBuilder.parse(inputSource);
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when parse xml: '%s - %s'", e.getClass().getName(), e.getMessage()));
				throw e;
			}
			Element commandelement = document.getDocumentElement();
			String commandtype = commandelement.getAttribute("type");
			if (! commandtype.equals("UPDATE_CONTENT")) {
				continue;
			}
			NodeList informationNodeList = commandelement.getElementsByTagName("information");
			Element informationelement = (Element) informationNodeList.item(0);
			String informationid = informationelement.getAttribute("id");
			if (informationid.equals(pInformationID)) {
				Log.i(TAG, String.format("Cancel command '%s' for delete content '%s'.", commandcacheinfo.getCommandID(), pInformationID));
				mCommandCacheDB.ForceFinishExecute(commandcacheinfo.getCommandID(), CommandExecuteStatus.ExecuteFailed, FormatErrorMessage("Command be canceled by delete content operation."));
			}
		}
	}
}
