/*!
 * Copyright (C) 2010-2013 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * GetStatusCommandProcessor
 * ȡϵͳ״ָ̬ࡣ
 * 
 * @version 0.01
 * @author tony (tonixinot@gmail.com)
 */

package com.kazovision.pvdaemon.communicate.commandprocessor;

import java.io.BufferedReader;
import java.io.FileReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.Locale;

import org.apache.http.conn.util.InetAddressUtils;
import org.w3c.dom.Element;

import android.content.Context;
import android.util.Log;

import com.kazovision.pvdaemon.Config;
import com.kazovision.pvdaemon.PvdaemonService;
import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.communicate.CommandCacheDB;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandExecuteStatus;
import com.kazovision.pvdaemon.download.DownloadController;

public class GetStatusCommandProcessor extends BaseCommandProcessor {
	private static final String TAG = "GetStatusCommandProcessor";

	public GetStatusCommandProcessor(Context pContext, CommandCacheDB pCommandCacheDB, DownloadController pDownloadController) {
		super(pContext, pCommandCacheDB, pDownloadController);
	}
	
	@Override
	public void ProcessCommand(Element pCommandElement, CommandExecuteStatus pExecuteStatus) {
		try {
			mExecuteStatus = CommandExecuteStatus.ExecuteSuccess;
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
			String systemtime = df.format(new java.util.Date());
			String starttime = df.format(PvdaemonService.GetStartTime());
			mReturnContents = "<status ";
			mReturnContents += "version=\"" + EscapeXmlString(Config.SYSTEM_BUILD_VERSION) + "\" ";
			mReturnContents += "terminalname=\"" + EscapeXmlString(GetTerminalName()) + "\" ";
			mReturnContents += "screenwidth=\"" + EscapeXmlString(GetScreenWidthInfo()) + "\" ";
			mReturnContents += "screenheight=\"" + EscapeXmlString(GetScreenHeightInfo()) + "\" ";
			mReturnContents += "screenwidth_server=\"" + EscapeXmlString(GetScreenWidthServerInfo()) + "\" ";
			mReturnContents += "screenheight_server=\"" + EscapeXmlString(GetScreenHeightServerInfo()) + "\" ";
			mReturnContents += "systemtime=\"" + EscapeXmlString(systemtime) + "\" ";
			mReturnContents += "starttime=\"" + EscapeXmlString(starttime) + "\" ";
			mReturnContents += "runningmode=\"" + EscapeXmlString(GetRunningModeInfo()) + "\" ";
			mReturnContents += "currentcontent=\"" + EscapeXmlString(GetCurrentContentInfo()) + "\" ";
			mReturnContents += "storedcontents=\"" + EscapeXmlString(GetStoredContentsInfo()) + "\" ";
			mReturnContents += "expiredcontents=\"" + EscapeXmlString(GetExpiredContentsInfo()) + "\" ";
			mReturnContents += "currentnotify=\"" + EscapeXmlString(GetCurrentNotifyInfo()) + "\" ";
			mReturnContents += "storednotifies=\"" + EscapeXmlString(GetStoredNotifiesInfo()) + "\" ";
			mReturnContents += "expirednotifies=\"" + EscapeXmlString(GetExpiredNotifiesInfo()) + "\" ";
			mReturnContents += "systeminfo=\"" + EscapeXmlString(GetSystemInfo()) + "\" ";
			mReturnContents += "/>";
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when process command '%s - %s'.", e.getClass().getName(), e.getMessage()));
			mExecuteStatus = CommandExecuteStatus.ExecuteFailed;
			mReturnContents = FormatErrorMessage(e.getClass().getName() + "-" + e.getMessage());
		}
	}
	
	private String GetTerminalName() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "terminalname");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String terminalname = bufferedreader.readLine();
			bufferedreader.close();
			if (terminalname == null) {
				terminalname = "";
			}
			return terminalname;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetScreenWidthInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "screenwidth");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String screenwidth = bufferedreader.readLine();
			bufferedreader.close();
			if (screenwidth == null) {
				screenwidth =  "";
			}
			return screenwidth;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetScreenHeightInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "screenheight");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String screenheight = bufferedreader.readLine();
			bufferedreader.close();
			if (screenheight == null) {
				screenheight = "";
			}
			return screenheight;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetScreenWidthServerInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "screenwidth_server");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String screenwidthserver = bufferedreader.readLine();
			bufferedreader.close();
			if (screenwidthserver == null) {
				screenwidthserver = "";
			}
			return screenwidthserver;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetScreenHeightServerInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "screenheight_server");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String screenheightserver = bufferedreader.readLine();
			bufferedreader.close();
			if (screenheightserver == null) {
				screenheightserver = "";
			}
			return screenheightserver;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetRunningModeInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "runningmode");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String runningmode = bufferedreader.readLine();
			bufferedreader.close();
			if (runningmode == null) {
				runningmode = "";
			}
			return runningmode;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetCurrentContentInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserSubPath("ram") + "currentcontent");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String currentcontent = bufferedreader.readLine();
			bufferedreader.close();
			if (currentcontent == null) {
				currentcontent = "";
			}
			return currentcontent;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetStoredContentsInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserSubPath("ram") + "storedcontents");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String storedcontents = bufferedreader.readLine();
			bufferedreader.close();
			if (storedcontents == null) {
				storedcontents = "";
			}
			return storedcontents;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetExpiredContentsInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserSubPath("ram") + "expiredcontents");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String expiredcontents = bufferedreader.readLine();
			bufferedreader.close();
			if (expiredcontents == null) {
				expiredcontents = "";
			}
			return expiredcontents;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetCurrentNotifyInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserSubPath("ram") + "currentnotify");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String currentnotify = bufferedreader.readLine();
			bufferedreader.close();
			if (currentnotify == null) {
				currentnotify = "";
			}
			return currentnotify;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetStoredNotifiesInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserSubPath("ram") + "storednotifies");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String storednotifies = bufferedreader.readLine();
			bufferedreader.close();
			if (storednotifies == null) {
				storednotifies = "";
			}
			return storednotifies;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetExpiredNotifiesInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserSubPath("ram") + "expirednotifies");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String expirednotifies = bufferedreader.readLine();
			bufferedreader.close();
			if (expirednotifies == null) {
				expirednotifies = "";
			}
			return expirednotifies;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetSystemInfo() {
		String systeminfo = "";
		
		//TODO: ȡϵͳϢ
		
		//IP
		String ipaddress = "";
		try {
			for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces(); en.hasMoreElements(); ) {
				NetworkInterface intf = en.nextElement();
				for (Enumeration<InetAddress> enIpAddr = intf.getInetAddresses(); enIpAddr.hasMoreElements(); ) {
					InetAddress inetAddress = enIpAddr.nextElement();
					if (inetAddress.isLoopbackAddress()) {
						continue;
					}
					if (! InetAddressUtils.isIPv4Address(inetAddress.getHostAddress())) {		//ɼIPv6ĵַ
						continue;
					}
					if (ipaddress.length() > 0) {
						ipaddress += ",";
					}
					ipaddress += inetAddress.getHostAddress().toString();
				}
			}
			
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when get ip: '%s %s'.", e.getClass().getName(), e.getMessage()));
			ipaddress = e.getClass().getName() + " - " + e.getMessage();
		}
		systeminfo += "IP:" + ipaddress + "; ";
		
		systeminfo += "STATUS:NORMAL";
		return systeminfo;
	}
}
