/*!
 * Copyright (C) 2010-2012 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * UpdateNotifyCommandProcessor
 * ָ֪ͨࡣ
 * 
 * @version 0.01
 * @author lyq (kazovision.liang@gmail.com)
 */

package com.kazovision.pvdaemon.communicate.commandprocessor;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import android.content.Context;
import android.util.Log;

import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.communicate.CommandCacheDB;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandExecuteStatus;
import com.kazovision.pvdaemon.download.DownloadController;
import com.kazovision.pvdaemon.playercontrol.PlayerController;

public class UpdateNotifyCommandProcessor extends BaseCommandProcessor {
	private static final String TAG = "UpdateNotifyCommandProcessor";
	
	public UpdateNotifyCommandProcessor(Context pContext, CommandCacheDB pCommandCacheDB, DownloadController pDownloadController) {
		super(pContext, pCommandCacheDB, pDownloadController);
	}

	@Override
	public void ProcessCommand(Element pCommandElement,CommandExecuteStatus pExecuteStatus) {
		try {
			NodeList informationlist = pCommandElement.getElementsByTagName("information");
			Element informationelement = (Element)informationlist.item(0);
			String informationid = informationelement.getAttribute("id");
			NodeList notifylist = pCommandElement.getElementsByTagName("notify");
			Element notifyelement = (Element)notifylist.item(0);
			String name = notifyelement.getAttribute("name");
			String text = notifyelement.getAttribute("text");
			Log.i(TAG, String.format("Update notify, name='%s', text='%s'.", name, text));
			
			//pvplayerĿ¼µnotify·
			UserPath userpath = new UserPath();
			String notifypath = userpath.GetUserSubPath("notify");
			
			//пû֪ͨͨҪԭʹͨĸ֪ͨɾ
			File dir = new File(notifypath);
			File file[] = dir.listFiles();
			for (int i = 0; i < file.length; i++) {
				if (file[i].isDirectory()){
					File notifyfile = new File(file[i].getAbsolutePath() + File.separator + informationid + ".notify");
					if (notifyfile.exists()) {
						if (! notifyfile.delete()) {
							Log.e(TAG, String.format("Error when delete notify file '%s'.", notifyfile));
						}
					}
					File infofile = new File(file[i].getAbsolutePath() + File.separator + informationid + ".info");
					if (infofile.exists()) {
						if (! infofile.delete()) {
							Log.e(TAG, String.format("Error when delete info file '%s'.", infofile));
						}
					}
				}
			}
			
			notifypath = userpath.GetUserSubPath("notify/" + name);
			//洢notifyļ
			String notifyfilename = notifypath + informationid + ".notify";
			File notifyfile = new File(notifyfilename);
			if (notifyfile.exists()) {
				if (! notifyfile.delete()) {
					Log.e(TAG, "Error when delete original notify.");
					throw new Exception("Error when delete original notify.");
				}
			}
			try {
				if (notifyfile.createNewFile()){
					FileWriter fileWriter = new FileWriter(notifyfile);
					BufferedWriter bufwriter = new BufferedWriter(fileWriter);
					bufwriter.write(text);
					bufwriter.flush();
					bufwriter.close();
					fileWriter.close();
				}
			} catch (IOException e) {
				Log.e(TAG, "Error when write notify file.");
				throw new Exception("Error when write notify file.");
			}
			
			//洢infoļ
			String infofilename =  notifypath + informationid + ".info";
			File infofile = new File(infofilename);
			if (infofile.exists()) {
				infofile.delete();
			}
			if (infofile.createNewFile()){
				//׼дļxml
				DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
				DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
				Document document = documentBuilder.newDocument();
				Element infoelement = document.createElement("info");
				document.appendChild(infoelement);
				infoelement.setAttribute("informationid", informationid);
				infoelement.setAttribute("textbrief", text.substring(0,9) + "...");
				if (notifyelement.getAttribute("datetype") == "DATE") {
					infoelement.setAttribute("datetype", "date");
					infoelement.setAttribute("startdate", notifyelement.getAttribute("startdate"));
					infoelement.setAttribute("stopdate", notifyelement.getAttribute("stopdate"));
				}
				else {
					infoelement.setAttribute("datetype", "always");
				}
				if (notifyelement.getAttribute("timetype") == "TIME") {
					infoelement.setAttribute("timetype", "time");
					infoelement.setAttribute("starttime", notifyelement.getAttribute("starttime"));
					infoelement.setAttribute("stoptime", notifyelement.getAttribute("stoptime"));
				}
				else {
					infoelement.setAttribute("timetype", "wholeday");
				}
				infoelement.setAttribute("priority", notifyelement.getAttribute("priority"));
				//д.infoļ
				try {
					TransformerFactory transFactory = TransformerFactory.newInstance();
					Transformer transformer = transFactory.newTransformer();
					transformer.transform(new DOMSource(document), new StreamResult(infofile));
				}
				catch (Exception e) {
					Log.e(TAG, String.format("Error when save info file: '%s - %s'.", e.getClass().getName(), e.getMessage()));
					throw new Exception("Error when save info file.");
				}
			}
			
			//֪ͨ
			PlayerController playercontroller = new PlayerController(mContext);
			playercontroller.ReloadNotify();
			
			mExecuteStatus = CommandExecuteStatus.ExecuteSuccess;
			mReturnContents = "";
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when process command '%s - %s'.", e.getClass().getName(), e.getMessage()));
			mExecuteStatus = CommandExecuteStatus.ExecuteFailed;
			mReturnContents = FormatErrorMessage(e.getClass().getName() + "-" + e.getMessage());
		}
	}
}
