/*!
 * Copyright (C) 2010-2013 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * UpdateScreenSizeCommandProcessor
 * ն˻ߴָ
 * 
 * @version 0.01
 * @author tony (tonixinot@gmail.com)
 */

package com.kazovision.pvdaemon.communicate.commandprocessor;

import java.io.BufferedWriter;
import java.io.FileWriter;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import android.content.Context;
import android.util.Log;

import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.communicate.CommandCacheDB;
import com.kazovision.pvdaemon.communicate.CommandCacheInfo.CommandExecuteStatus;
import com.kazovision.pvdaemon.download.DownloadController;
import com.kazovision.pvdaemon.playercontrol.PlayerController;

public class UpdateScreenSizeCommandProcessor  extends BaseCommandProcessor{
	private static final String TAG = "UpdateScreenSizeCommandProcessor";

	public UpdateScreenSizeCommandProcessor(Context pContext, CommandCacheDB pCommandCacheDB, DownloadController pDownloadController){
		super(pContext, pCommandCacheDB, pDownloadController);
	}

	@Override
	public void ProcessCommand(Element pCommandElement,CommandExecuteStatus pExecuteStatus) {
		try {
			NodeList nodelist = pCommandElement.getElementsByTagName("screensize");
			if (nodelist.getLength() == 0) {
				throw new Exception("Screen size element not exists.");
			}
			Element screensizeelement = (Element)nodelist.item(0);
			String widthstr = screensizeelement.getAttribute("width");
			String heightstr = screensizeelement.getAttribute("height");
			if (widthstr.length() > 0 && heightstr.length() > 0) {
				int width = Integer.parseInt(widthstr);
				int height = Integer.parseInt(heightstr);
				if (width <= 0 || height <= 0) {
					Log.w(TAG, "Screen size not correct.");
					throw new Exception("Screen size not correct.");
				}
			}
			SetScreenWidthInfo(widthstr);
			SetScreenHeightInfo(heightstr);
			SetScreenWidthServerInfo(widthstr);
			SetScreenHeightServerInfo(heightstr);
			
			//֪ͨ
			PlayerController playercontroller = new PlayerController(mContext);
			playercontroller.ReloadScreenSize();
			
			Log.i(TAG, String.format("Screen size been modified '%sx%s'.", widthstr, heightstr));
			mExecuteStatus = CommandExecuteStatus.ExecuteSuccess;
			mReturnContents = "";
		}
		catch (Exception e) {
			Log.e(TAG, String.format("Error when process command '%s - %s'.", e.getClass().getName(), e.getMessage()));
			mExecuteStatus = CommandExecuteStatus.ExecuteFailed;
			mReturnContents = FormatErrorMessage(e.getClass().getName() + "-" + e.getMessage());
		}
	}
	
	private void SetScreenWidthInfo(String pScreenWidth) throws Exception {
		UserPath userpath = new UserPath();
		FileWriter filewriter = new FileWriter(userpath.GetUserPath() + "screenwidth");
		BufferedWriter bufferedwriter = new BufferedWriter(filewriter);
		bufferedwriter.write(pScreenWidth);
		bufferedwriter.flush();
		bufferedwriter.close();
	}
	
	private void SetScreenHeightInfo(String pScreenHeight) throws Exception {
		UserPath userpath = new UserPath();
		FileWriter filewriter = new FileWriter(userpath.GetUserPath() + "screenheight");
		BufferedWriter bufferedwriter = new BufferedWriter(filewriter);
		bufferedwriter.write(pScreenHeight);
		bufferedwriter.flush();
		bufferedwriter.close();
	}
	
	private void SetScreenWidthServerInfo(String pScreenWidthServer) throws Exception {
		UserPath userpath = new UserPath();
		FileWriter filewriter = new FileWriter(userpath.GetUserPath() + "screenwidth_server");
		BufferedWriter bufferedwriter = new BufferedWriter(filewriter);
		bufferedwriter.write(pScreenWidthServer);
		bufferedwriter.flush();
		bufferedwriter.close();
	}
	
	private void SetScreenHeightServerInfo(String pScreenHeightServer) throws Exception {
		UserPath userpath = new UserPath();
		FileWriter filewriter = new FileWriter(userpath.GetUserPath() + "screenheight_server");
		BufferedWriter bufferedwriter = new BufferedWriter(filewriter);
		bufferedwriter.write(pScreenHeightServer);
		bufferedwriter.flush();
		bufferedwriter.close();
	}
}
