/*!
 * Copyright (C) 2010-2013 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * DownloadController
 * ؿࡣ
 * 
 * @version 0.01
 * @author lianghong (lh_ann@163.com)
 */

package com.kazovision.pvdaemon.download;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import android.content.Context;
import android.util.Log;

import com.kazovision.pvdaemon.download.DownloadInfo.FileDownloadStatus;

public class DownloadController {
	private static final String TAG = "DownloadController";
	private DownloadThread mDownloadThread;
	private DownloadNotifyThread mDownloadNotifyThread;
	private DownloadDB mDownloadDB;
	private List<DownloadInfo> mDownloadInfoList;
	
	public DownloadController(Context pContext) {
		mDownloadThread = new DownloadThread(this);
		mDownloadNotifyThread = new DownloadNotifyThread(this);
		mDownloadDB = new DownloadDB(pContext);
		mDownloadInfoList = new ArrayList<DownloadInfo>();
	}
	
	public DownloadInfo BeginDownloadFile() {
		DownloadInfo downloadinfo = null;
		boolean found = false;
		for (int i=0; i<mDownloadInfoList.size(); i++) {
			downloadinfo = mDownloadInfoList.get(i);
			//ļ֮ǰʧˣĿʼʱ佫ᱻóδĳʱ㣬ʱļ̱½ء
			Date date = new java.util.Date();
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
			try {
				if (df.parse(downloadinfo.GetNextDownloadTime()).after(date)) {
					continue;
				}
			} catch (ParseException e) {
				Log.e(TAG, e.getStackTrace().toString());
			}
			found = true;
			mDownloadInfoList.remove(i);
			break;
		}
		if (!found) {
			return null;
		}
		downloadinfo.SetDownloadStatus(FileDownloadStatus.Downloading);
		mDownloadDB.UpdateDownloadInfoDownloadStatus(downloadinfo);
		return downloadinfo;
	}
	
	public void UpdateDownloadFileStatus(DownloadInfo pDownloadInfo) {
		mDownloadDB.UpdateDownloadInfoDownloadStatus(pDownloadInfo);
		if (pDownloadInfo.GetDownloadStatus() == FileDownloadStatus.DownloadTemporaryFailed){		//½Ϣļбȴ´ء
			mDownloadInfoList.add(pDownloadInfo);
		}
	}
	
	public List<DownloadInfo> GetWaitingForNotifyDownloadInfoList() {
		return mDownloadDB.GetWaitingForNotifyDownloadInfoList();
	}
	
	public void SetDownloadInfoNotified(int pID) {
		mDownloadDB.SetDownloadInfoNotified(pID);
	}
	
	public void ClearUselessDownloadInfo() {
		List<DownloadInfo> downloadinfolist;
		downloadinfolist = mDownloadDB.GetUselessDownloadInfoList();
		for (DownloadInfo downloadinfo: downloadinfolist) {
			File file = new File(downloadinfo.GetLocalFileName());
			if (file.exists()){
				file.delete();
			}
			mDownloadDB.RemoveDownloadInfo(downloadinfo.GetID());
		}
	}
	
	public boolean DownloadFile(String pModuleName, String pFileUrl, String pMd5) {
		DownloadInfo downloadinfo;
		downloadinfo = mDownloadDB.FindDownloadInfo(pFileUrl);
		if (downloadinfo != null) { //ļѾбˡ
			if (downloadinfo.GetDownloadStatus() == FileDownloadStatus.DownloadSucceed || downloadinfo.GetDownloadStatus() == FileDownloadStatus.DownloadFailed
				|| downloadinfo.GetMd5().compareTo(pMd5) != 0) {//ļؽˣMD5ԭĲͬʱҪظļ
				File file = new File(downloadinfo.GetLocalFileName());
				if (file.exists()) {
					file.delete();
				}
			}
			mDownloadDB.RemoveDownloadInfo(downloadinfo.GetID());
			//һ
			downloadinfo = mDownloadDB.AppendDownloadInfo(pModuleName, pFileUrl, pMd5);
			mDownloadInfoList.add(downloadinfo);
			return true;
		}
		else {
			downloadinfo = mDownloadDB.AppendDownloadInfo(pModuleName, pFileUrl, pMd5);
		}
		mDownloadInfoList.add(downloadinfo);
		Log.i(TAG, String.format("Add file '%s' to the download list.", pFileUrl));
		return true;
	}
	
	public void RegisterDownloadHelper(DownloadHelper pDownloadHelper) {
		if (! mDownloadNotifyThread.AppendDownloadHelper(pDownloadHelper)) {
			Log.e(TAG, "AppendDownloadHelper Error");
			return;
		}
	}
	
	public void Start() {
		mDownloadThread.start();
		mDownloadNotifyThread.start();
	}
	
	public void Stop() {
		mDownloadThread.interrupt();
		mDownloadNotifyThread.interrupt();
	}
}
