/*!
 * Copyright (C) 2010-2012 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * DownloadDB
 * ļݿ⡣
 * 
 * @version 0.01
 * @author lyq (kazovision.liang@gmail.com)
 */

package com.kazovision.pvdaemon.download;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.UUID;

import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;

import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.download.DownloadInfo.DownloadNotifyStatus;
import com.kazovision.pvdaemon.download.DownloadInfo.FileDownloadStatus;

public class DownloadDB {
	private static final String TAG = "DownloadDB";
	private static final int DOWNLOAD_FINISHED_AND_NOTIFIED_INFO_REMOVE_TIME = 5;
	private static final int DOWNLOAD_FINISHED_INFO_REMOVE_TIME = 8;
	private static final int DOWNLOAD_UNFINISHED_INFO_REMOVE_TIME = 24;
	
	private class DownloadDBHelper extends SQLiteOpenHelper {
		public DownloadDBHelper(Context pContext) {
			super(pContext, "download", null, 1);
		}
		
		@Override
		public void onCreate(SQLiteDatabase pDatabase) {
			pDatabase.execSQL("CREATE TABLE 'download' ("
					+ "'id' INTEGER PRIMARY KEY,"
					+ "'modulename' VARCHAR,"
					+ "'fileurl' VARCHAR,"
					+ "'md5' VARCHAR,"
					+ "'downloadstarttime' VARCHAR,"
					+ "'localfilename' VARCHAR,"
					+ "'nextdownloadtime' VARCHAR,"
					+ "'downloadstatus' INTEGER,"
					+ "'downloadprogress' INTEGER,"
					+ "'downloadfinishedtime' VARCHAR,"
					+ "'notifystatus' INTEGER)");
		}
		
		@Override
		public void onUpgrade(SQLiteDatabase pDatabase, int pOldVersion, int pNewVersion) {
			pDatabase.execSQL("DROP TABLE IF EXISTS 'download')");
			onCreate(pDatabase);
		}
	};

	private Context mContext;
	private SQLiteDatabase mDatabase;
	
	public DownloadDB(Context pContext) {
		mContext = pContext;
		DownloadDBHelper helper = new DownloadDBHelper(pContext);
		mDatabase = helper.getWritableDatabase();
	}
	
	public DownloadInfo FindDownloadInfo(String pFileUrl){
		DownloadInfo downloadInfo = null;
		Cursor cursor = null;
		
		try {
			String sql = "SELECT id, modulename, fileurl, md5, downloadstarttime, localfilename, nextdownloadtime, downloadstatus, downloadprogress, downloadfinishedtime, notifystatus "
					+ "FROM download WHERE fileurl = ?";
			String[] params = new String[]{pFileUrl};
			cursor = mDatabase.rawQuery(sql, params);
			if (cursor.moveToNext()) {
				downloadInfo = new DownloadInfo();
				FillDownloadInfo(cursor, downloadInfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return downloadInfo;	
	}
	
	public DownloadInfo AppendDownloadInfo(String pModuleName, String pFileUrl, String pMd5)
	{
		String sql ="";
		DownloadInfo downloadInfo = new DownloadInfo();
		
		downloadInfo.SetID(-1);
		downloadInfo.SetModuleName(pModuleName);
		downloadInfo.SetFileUrl(pFileUrl);
		downloadInfo.SetMd5(pMd5);
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
		String downloadStartTime = df.format(new java.util.Date());
		downloadInfo.SetDownloadStartTime(downloadStartTime);
		UUID uuid = UUID.randomUUID(); 
		UserPath userpath = new UserPath();
		String localfilename = userpath.GetUserSubPath("download") + uuid.toString() + ".tmp";
		downloadInfo.SetLocalFileName(localfilename); 
		downloadInfo.SetNextDownloadTime(downloadStartTime);
		downloadInfo.SetDownloadStatus(FileDownloadStatus.WaitingForDownload);
		downloadInfo.SetDownloadProgress(0);
		downloadInfo.SetDownloadFinishedTime(downloadStartTime);
		downloadInfo.SetNotifyStatus(DownloadNotifyStatus.Notified);
		
		sql = "INSERT INTO download (modulename, fileurl, md5, downloadstarttime, localfilename, nextdownloadtime,"
				+ " downloadstatus, downloadprogress, downloadfinishedtime, notifystatus) "
				+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
		Object[] params = new Object[]{downloadInfo.GetModuleName(), downloadInfo.GetFileUrl(), downloadInfo.GetMd5(), downloadInfo.GetDownloadStartTime(), 
				downloadInfo.GetLocalFileName(), downloadInfo.GetNextDownloadTime(), downloadInfo.GetDownloadStatus().ordinal(), downloadInfo.GetDownloadProgress(),
				downloadInfo.GetDownloadFinishedTime(), downloadInfo.GetNotifyStatus().ordinal()};
		mDatabase.execSQL(sql, params);

		//ѯݵID
		Cursor cursor = null;
		try {
			sql = "SELECT last_insert_rowid() FROM download";
			cursor = mDatabase.rawQuery(sql, null);
			if (cursor.moveToFirst()) {
				downloadInfo.SetID(cursor.getInt(0));
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return downloadInfo;
	}
	
	public void RemoveDownloadInfo(int pID){
		String sql = "DELETE FROM download WHERE id = ?";
		Object[] params = new Object[]{pID};
		mDatabase.execSQL(sql, params);
	}
	
	public void UpdateDownloadInfoDownloadStatus(DownloadInfo pDownloadInfo){
		String sql = "UPDATE download SET nextdownloadtime = ?, downloadstatus = ?, downloadprogress = ?, downloadfinishedtime = ?, notifystatus = ? WHERE id = ?";
		Object[] params = new Object[]{pDownloadInfo.GetNextDownloadTime(),pDownloadInfo.GetDownloadStatus().ordinal(),pDownloadInfo.GetDownloadProgress(),
				pDownloadInfo.GetDownloadFinishedTime(),DownloadNotifyStatus.WaitingForNotify.ordinal(),pDownloadInfo.GetID()};
		mDatabase.execSQL(sql, params);
	}
	
	public List<DownloadInfo> GetUnfinishedDownloadInfoList(){
		List<DownloadInfo> downloadinfolist = new ArrayList<DownloadInfo>();
		Cursor cursor = null;
		try {
			String sql = "SELECT id, modulename, fileurl, md5, downloadstarttime, localfilename, nextdownloadtime,"
					+ " downloadstatus, downloadprogress, downloadfinishedtime, notifystatus "
					+ "FROM download "
					+ "WHERE downloadstatus = ? OR downloadstatus = ? OR downloadstatus = ?";
			String[] params = new String[]{Integer.toString(FileDownloadStatus.WaitingForDownload.ordinal()),
					Integer.toString(FileDownloadStatus.Downloading.ordinal()),
					Integer.toString(FileDownloadStatus.DownloadTemporaryFailed.ordinal())};
			cursor = mDatabase.rawQuery(sql, params);
			while (cursor.moveToNext()) {
				DownloadInfo downloadinfo = new DownloadInfo();
				FillDownloadInfo(cursor, downloadinfo);
				downloadinfolist.add(downloadinfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return downloadinfolist;
	}
	
	public List<DownloadInfo> GetWaitingForNotifyDownloadInfoList(){
		List<DownloadInfo> downloadinfolist = new ArrayList<DownloadInfo>();
		//ȡ֪ͨ״̬Ϊ WaitingForNotify  Notifying Ϣ
		Cursor cursor = null;
		try {
			String sql = "SELECT id, modulename, fileurl, md5, downloadstarttime, localfilename, nextdownloadtime,"
					+ " downloadstatus, downloadprogress, downloadfinishedtime, notifystatus "
					+ "FROM download "
					+ "WHERE notifystatus = ? OR notifystatus = ? ";
			String[] params = new String[]{Integer.toString(DownloadNotifyStatus.WaitingForNotify.ordinal()),
					Integer.toString(DownloadNotifyStatus.Notifying.ordinal())};
			cursor = mDatabase.rawQuery(sql, params);
			while (cursor.moveToNext()) {
				DownloadInfo downloadinfo = new DownloadInfo();
				FillDownloadInfo(cursor, downloadinfo);
				downloadinfolist.add(downloadinfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		
		//Ϣ֪ͨ״̬Ϊ Notifying
		String sql = "UPDATE download SET notifystatus = ? WHERE notifystatus = ? OR notifystatus = ?";
		Object[] params = new Object[]{DownloadNotifyStatus.Notifying.ordinal(),DownloadNotifyStatus.WaitingForNotify.ordinal(),DownloadNotifyStatus.Notifying.ordinal()};
		mDatabase.execSQL(sql, params);
		
		return downloadinfolist;
	}
	
	public void SetDownloadInfoNotified(int pID)
	{
		String sql = "UPDATE download SET notifystatus = ? WHERE id = ? AND notifystatus = ?";
		Object[] params = new Object[]{DownloadNotifyStatus.Notified.ordinal(),pID,DownloadNotifyStatus.Notifying.ordinal()};
		mDatabase.execSQL(sql, params);
	}
	
	public List<DownloadInfo> GetUselessDownloadInfoList(){
		List<DownloadInfo> downloadinfolist = new ArrayList<DownloadInfo>();
		//ȡ֪ͨ״̬Ϊ WaitingForNotify  Notifying Ϣ
		Cursor cursor = null;
		try {
			String sql = "SELECT id, modulename, fileurl, md5, downloadstarttime, localfilename, nextdownloadtime, "
					+ " downloadstatus, downloadprogress, downloadfinishedtime, notifystatus "
					+ "FROM download "
					+ "WHERE ((downloadstatus = ? OR downloadstatus = ?) AND (notifystatus = ?) AND (downloadfinishedtime < ?)) "
					+ "OR ((downloadstatus = ? OR downloadstatus = ?) AND (downloadfinishedtime < ?)) "
					+ "OR ((downloadstatus <> ? AND downloadstatus <> ?) AND (downloadstarttime < ?))";
			
			Date date = new java.util.Date();
			long time = date.getTime();
			//time = time - DOWNLOAD_FINISHED_AND_NOTIFIED_INFO_REMOVE_TIME * 60*60*1000;
			SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
			Date date1 = new Date(time - DOWNLOAD_FINISHED_AND_NOTIFIED_INFO_REMOVE_TIME * 60*60*1000);
			Date date2 = new Date(time - DOWNLOAD_FINISHED_INFO_REMOVE_TIME * 60*60*1000);
			Date date3 = new Date(time - DOWNLOAD_UNFINISHED_INFO_REMOVE_TIME * 60*60*1000);
			
			String[] params = new String[]{
					//ؽ֪ͨʱ䳬һʱ䡣
					Integer.toString(FileDownloadStatus.DownloadSucceed.ordinal()),
					Integer.toString(FileDownloadStatus.DownloadFailed.ordinal()),
					Integer.toString(DownloadNotifyStatus.Notified.ordinal()),
					df.format(date1),
					//ؽʱ䳬һʱ䡣
					Integer.toString(FileDownloadStatus.DownloadSucceed.ordinal()),
					Integer.toString(FileDownloadStatus.DownloadFailed.ordinal()),
					df.format(date2),
					//δؽؿʼʱ䳬һʱ䡣
					Integer.toString(FileDownloadStatus.DownloadSucceed.ordinal()),
					Integer.toString(FileDownloadStatus.DownloadFailed.ordinal()),
					df.format(date3)};
			cursor = mDatabase.rawQuery(sql, params);
			while (cursor.moveToNext()) {
				DownloadInfo downloadinfo = new DownloadInfo();
				FillDownloadInfo(cursor, downloadinfo);
				downloadinfolist.add(downloadinfo);
			}
		}
		finally {
			if (cursor != null) {
				cursor.close();
			}
		}
		return downloadinfolist;
	}
	
	public void FillDownloadInfo(Cursor pCur, DownloadInfo pDownloadInfo) {
		pDownloadInfo.SetID(pCur.getInt(pCur.getColumnIndex("id")));
		pDownloadInfo.SetModuleName(pCur.getString(pCur.getColumnIndex("modulename")));
		pDownloadInfo.SetFileUrl(pCur.getString(pCur.getColumnIndex("fileurl")));
		pDownloadInfo.SetMd5(pCur.getString(pCur.getColumnIndex("md5")));
		pDownloadInfo.SetDownloadStartTime(pCur.getString(pCur.getColumnIndex("downloadstarttime")));
		pDownloadInfo.SetLocalFileName(pCur.getString(pCur.getColumnIndex("localfilename")));
		pDownloadInfo.SetNextDownloadTime(pCur.getString(pCur.getColumnIndex("nextdownloadtime")));
		pDownloadInfo.SetDownloadStatus(FileDownloadStatus.values()[pCur.getInt(pCur.getColumnIndex("downloadstatus"))]);
		pDownloadInfo.SetDownloadProgress(pCur.getInt(pCur.getColumnIndex("downloadprogress")));
		pDownloadInfo.SetDownloadFinishedTime(pCur.getString(pCur.getColumnIndex("downloadfinishedtime")));
		pDownloadInfo.SetNotifyStatus(DownloadNotifyStatus.values()[pCur.getInt(pCur.getColumnIndex("notifystatus"))]);
	}
}
