/*!
 * Copyright (C) 2010-2013 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * DownloadThread
 * ̡߳
 * 
 * @version 0.01
 * @author lianghong (lh_ann@163.com)
 */

package com.kazovision.pvdaemon.download;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import android.util.Log;

import com.kazovision.pvdaemon.common.MD5;
import com.kazovision.pvdaemon.download.DownloadInfo.FileDownloadStatus;

public class DownloadThread extends Thread {
	private static final String TAG = "DownloadThread";
	private static final int DOWNLOAD_FAILED_RETRY_INTERVAL = 5*60;
	private DownloadController mDownloadController;
	private DownloadInfo mDownloadInfo;
	
	public DownloadThread (DownloadController pDownloadController) {
		mDownloadController = pDownloadController;
	}
	
	@Override
	public void start() {
		setName("Download Thread");
		super.start();
	}
	
	public void run() {
		while(! interrupted()) {
			mDownloadInfo = mDownloadController.BeginDownloadFile();
			//mDownloadInfoΪʱµ
			if (mDownloadInfo == null) {
				try {
					sleep(10000);
					continue;
				} catch (InterruptedException e) {
					Log.e(TAG, "Download thread inturrepted.");
					break;
				}
			}
			if (mDownloadInfo != null) {
				boolean downloadsuccess = false;
				String fileurl = mDownloadInfo.GetFileUrl();
				String filename = mDownloadInfo.GetLocalFileName();
				Log.v(TAG, String.format("Downloading file '%s'.", fileurl));
				//ļ߳
				try {
					URL url = new URL(fileurl);
					URLConnection urlconnection = url.openConnection();
					InputStream inputstream = urlconnection.getInputStream();
					byte[] stream = new byte[1024];
					int length;
					OutputStream outputstream = new FileOutputStream(filename);
					while ((length = inputstream.read(stream)) != -1) {
						outputstream.write(stream, 0, length);
					}
					outputstream.close();
					inputstream.close();
					downloadsuccess = true;
				} catch (Exception e) {
					Log.e(TAG, String.format("Download procedure be canceled. fileurl='%s'.", fileurl));
				}
				if (downloadsuccess) {
					if (mDownloadInfo.GetMd5().length() > 0 ) {
						//ļMD5ֵǷȷ
						if (CheckFileMd5(mDownloadInfo.GetLocalFileName(), mDownloadInfo.GetMd5())) {
							Log.i(TAG, "File md5 correct.");
						}
						else {
							Log.e(TAG, "File md5 not correct.");
							//ɾصļسΪļҪˡ
							File file2 = new File(mDownloadInfo.GetLocalFileName());
							if (file2.exists()) {
								if (! file2.delete()) {
									Log.e(TAG, String.format("Error when remove downloaded file '%s'", mDownloadInfo.GetLocalFileName()));
								}
							}
							downloadsuccess = false;
						}
					}
					else {
						Log.i(TAG, "No need to check md5.");
					}
				}
				if (downloadsuccess) {
					mDownloadInfo.SetDownloadStatus(FileDownloadStatus.DownloadSucceed);
					mDownloadInfo.SetDownloadProgress(100);
					SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
					String downloadFinishedTime = df.format(new java.util.Date());
					try {
						mDownloadInfo.SetDownloadFinishedTime(downloadFinishedTime);
					} catch (Exception e) {
						Log.e(TAG, String.format("Error when parse date time '%s - %s'.", e.getClass().getName(), e.getMessage()));
					}
				}
				else {
					Log.e(TAG, String.format("Download '%s' failed.", mDownloadInfo.GetFileUrl()));
					Date date;
					long time;
					SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.ENGLISH);
					try {
						date = df.parse(mDownloadInfo.GetDownloadStartTime());
						//ѳ1죬Ϊسʧˡ
						time = date.getTime();
						date=new Date(time + 24*60*60*1000);
						if (date.before(new java.util.Date())) {
							mDownloadInfo.SetDownloadStatus(FileDownloadStatus.DownloadFailed);
						}
						else {		//һʱ³ء
							date = new java.util.Date();
							time = date.getTime();
							date=new Date(time + DOWNLOAD_FAILED_RETRY_INTERVAL*1000);
							mDownloadInfo.SetNextDownloadTime(df.format(date));
							mDownloadInfo.SetDownloadStatus(FileDownloadStatus.DownloadTemporaryFailed);
						}
					} catch (ParseException e) {
						Log.e(TAG, e.getStackTrace().toString());
					}
				}
				mDownloadController.UpdateDownloadFileStatus(mDownloadInfo);
			}
		}
	}
	
	public boolean CheckFileMd5(String pFileName, String pExpectMd5) {
		File file = new File(pFileName); 
		String md5 = "";
		try {
			md5 = MD5.getFileMD5String(file);
		} catch (IOException e) {
			Log.e(TAG, e.getStackTrace().toString());
		}
		return md5.compareTo(pExpectMd5) == 0;
	}
}
