/*!
 * Copyright (C) 2010-2013 Kazo Vision. (http://www.kazovision.com)
 * All rights reserved.
 * 
 * NetworkHandler
 * ƴࡣ
 * 
 * @version 0.01
 * @author tony (tonixinot@gmail.com)
 */

package com.kazovision.pvdaemon.externalcontrol;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.RandomAccessFile;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Calendar;
import java.util.Date;

import android.content.Context;
import android.util.Log;

import com.kazovision.pvdaemon.common.TempPath;
import com.kazovision.pvdaemon.common.UserPath;
import com.kazovision.pvdaemon.fileimport.ConfigureFileImporter;
import com.kazovision.pvdaemon.fileimport.PackageFileImporter;

public class NetworkHandler {
	private static final String TAG = "NetworkHandler";
	private ServerConnectionThread mServerConnectionThread = null;
	
	class PackageLoader implements Runnable {
		private Context mContext;
		private String mInformationID;
		private String mPackageFile;
		
		public PackageLoader(Context pContext, String pInformationID, String pPackageFile) {
			mContext = pContext;
			mInformationID = pInformationID;
			mPackageFile = pPackageFile;
		}
		
		@Override
		public void run() {
			try {
				PackageFileImporter importer = new PackageFileImporter(mContext);
				Date publishtime = Calendar.getInstance().getTime();
				importer.LoadFromPackageFile(mInformationID, "manually", publishtime, mPackageFile);
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when load package: '%s - %s'.", e.getClass().getName(), e.getMessage()));
			}
		}
	}
	
	class ConfigureLoader implements Runnable {
		private Context mContext;
		private String mConfigureFile;
		
		public ConfigureLoader(Context pContext, String pConfigureFile) {
			mContext = pContext;
			mConfigureFile = pConfigureFile;
		}
		
		@Override
		public void run() {
			try {
				ConfigureFileImporter importer = new ConfigureFileImporter(mContext);
				importer.LoadFromConfigureFile(mConfigureFile);
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when load configure: '%s - %s'.", e.getClass().getName(), e.getMessage()));
			}
		}
	}
	
	class UpgradeLoader implements Runnable {
		private Context mContext;
		private String mUpgradeFile;
		
		public UpgradeLoader(Context pContext, String pUpgradeFile) {
			mContext = pContext;
			mUpgradeFile = pUpgradeFile;
		}
		
		@Override
		public void run() {
			//TODO: implement this.
			/*
			try {
				UpgradeFileImporter importer = new UpgradeFileImporter(mContext);
				importer.LoadFromConfigureFile(mUpgradeFile);
				Toast toast = Toast.makeText(mContext, "Upgrade been loaded.", Toast.LENGTH_LONG);
				toast.show();
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when load upgrade: '%s - %s'.", e.getClass().getName(), e.getMessage()));
				Toast toast = Toast.makeText(mContext, "Error when load upgrade.", Toast.LENGTH_LONG);
				toast.show();
			}
			*/
		}
	}
	
	class LoadFileCache {
		private Context mContext;
		private TempPath mTempPath = null;
		private RandomAccessFile mFile = null;
		
		public LoadFileCache(Context pContext) {
			mContext = pContext;
			try {
				mTempPath = new TempPath();
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when create temp path '%s - %s'.", e.getClass().getName(), e.getMessage()));
			}
		}
		
		public void ProcessCommand(byte[] pData, int pDataLength) throws Exception {
			if (pDataLength < 4) {
				throw new Exception("No enough data.");
			}
			int datapos = (pData[0] & 0xFF) | (pData[1] & 0xFF) << 8 | (pData[2] & 0xFF) << 16 | (pData[3] & 0xFF) << 24;
			if (datapos == -1) {		//ʼĿȡĿơ
				Log.i(TAG, "Begin upload file.");
				if (mFile != null) {
					mFile.close();
					mFile = null;
				}
				mFile = new RandomAccessFile(mTempPath.GetTempPath() + "temp.cache", "rw");
			}
			else if (datapos == -2) {		//Ŀмء
				Log.i(TAG, "Finish upload file.");
				if (mFile == null) {
					throw new Exception("File not opened yet.");
				}
				mFile.close();
				mFile = null;
				
				//̨ļ̣߳õǰִϣԱ㽫ظ༭ˡ\n
				//ҪȴļؽٷؽĻ༭˿ܻȴʱ
				int filetype = pData[4] & 0xFF;
				if (filetype == 1) {
					String informationid = new String(pData, 5, pDataLength-5);
					Thread thread = new Thread(new PackageLoader(mContext, informationid, mTempPath.GetTempPath() + "temp.cache"));
					thread.start();
				}
				else if (filetype == 2) {
					Thread thread = new Thread(new ConfigureLoader(mContext, mTempPath.GetTempPath() + "temp.cache"));
					thread.start();
				}
				else if (filetype == 3) {
					Thread thread = new Thread(new UpgradeLoader(mContext, mTempPath.GetTempPath() + "temp.cache"));
					thread.start();
				}
				else {
					throw new Exception(String.format("Unknown file type '%s'", filetype));
				}
			}
			else {
				if (mFile == null) {
					throw new Exception("File not opened yet.");
				}
				if (mFile.getFilePointer() != datapos) {
					throw new Exception("File position not correct.");
				}
				if (pDataLength < 5) {
					throw new Exception("No enough data.");
				}
				mFile.write(pData, 4, pDataLength-4);
			}
		}
	}
	
	class ClientConnection implements Runnable {
		private Socket mSocket;
		private LoadFileCache mLoadFileCache;
		
		public ClientConnection(Context pContext, Socket pSocket) {
			mSocket = pSocket;
			mLoadFileCache = new LoadFileCache(pContext);
		}
		
		@Override
		public void run() {
			Log.i(TAG, String.format("New connection from '%s'.", mSocket.getInetAddress().getHostAddress()));
			try {
				mSocket.setKeepAlive(true);
				mSocket.setSoLinger(true, 2);	//÷ͣʱ2롣
				mSocket.setSoTimeout(0);		//InputStreamϵread()ʱʱΪʱ
				mSocket.setTcpNoDelay(true);	//رNagle㷨
				InputStream inputstream = mSocket.getInputStream();
				byte[] buffer = new byte[65536];
				while (mSocket.isConnected()) {
					//ȡͷ
					if (inputstream.read(buffer, 0, 1) != 1) {
						break;		//һϿ˲Żᷢʱͽ̡߳
					}
					//ͷǷȷ
					if (buffer[0] != (byte)0xFF) {
						Log.e(TAG, String.format("Wrong header '%d'.", buffer[0] & 0xFF));
						continue;
					}
					//ȡָš
					if (inputstream.read(buffer, 0, 1) != 1) {
						Log.e(TAG, "Error when read command id.");
						continue;
					}
					int commandid = buffer[0] & 0xFF;
					//ȡݳȡ
					if (inputstream.read(buffer, 0, 2) != 2) {
						Log.e(TAG, "Error when read command length.");
						continue;
					}
					int commandlength = ((buffer[1] & 0xFF) << 8) | (buffer[0] & 0xFF);
					//ȡݡ
					int readedsize = 0;
					while (readedsize < commandlength + 1) {
						int curreadedsize = inputstream.read(buffer, readedsize, commandlength + 1 - readedsize);
						if (curreadedsize < 0) {
							Log.e(TAG, "Error when read buffer.");
							break;
						}
						readedsize += curreadedsize;
					}
					if (readedsize != commandlength + 1) {
						Log.e(TAG, "Not enough data be readed.");
						continue;
					}
					//βǷȷ
					if (buffer[commandlength] != (byte)0xFE) {
						Log.e(TAG, String.format("Received the wrong tail '%d'.", buffer[commandlength] & 0xFF));
						continue;
					}
					//յݡ
					byte[] result = null;
					try {
						result = ProcessCommand(commandid, buffer, commandlength);
						if (result == null) {
							result = new byte[1];
							result[0] = 0x01;
						}
					}
					catch (Exception e) {
						Log.e(TAG, String.format("Error when process command '%s - %s'.", e.getClass().getName(), e.getMessage()));
						result = new byte[1];
						result[0] = 0x00;
					}
					OutputStream outputstream = mSocket.getOutputStream();
					byte[] resultbuffer = new byte[5 + result.length];
					resultbuffer[0] = (byte) 0xFF;
					resultbuffer[1] = (byte) commandid;
					resultbuffer[2] = (byte) (result.length & 0xFF);
					resultbuffer[3] = (byte) (result.length >> 8 & 0xFF);
					for (int i=0; i<result.length; i++) {
						resultbuffer[4 + i] = result[i];
					}
					resultbuffer[4 + result.length] = (byte) 0xFE;
					outputstream.write(resultbuffer);
				}
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when read process network control: '%s - %s'.", e.getClass().getName(), e.getMessage()));
			}
			finally {
				if (mSocket != null) {
					try {
						mSocket.close();
					}
					catch (Exception e) {
						Log.e(TAG, "Error when close socket.");
					}
				}
			}
		}
		
		public byte[] ProcessCommand(int pCommandID, byte[] pData, int pDataLength) throws Exception {
			byte[] result = null;
			switch (pCommandID) {
			case 0x00:
				//Log.i(TAG, "Query version..");
				String systeminfo = "PVPLAYER" + "\n" + "0.01" + "\n" + GetMacInfo() + "\n" + GetTerminalNameInfo();
				byte[] systeminfobuffer = systeminfo.getBytes();
				result= systeminfobuffer;
				break;
			case 0x01:
				//Log.i(TAG, "Load package from stream.");
				mLoadFileCache.ProcessCommand(pData, pDataLength);
				break;
			case 0x02:
				{
				Log.i(TAG, "Load package from file.");
				String filename = new String(pData, 0, pDataLength, "UTF-8");
				//TODO: implement this.
				}
				break;
			case 0x03:
				{
				Log.i(TAG, "Load configure from file.");
				String filename = new String(pData, 0, pDataLength, "UTF-8");
				//TODO: implement this.
				}
				break;
			case 0x04:
				{
				Log.i(TAG, "Load upgrade from file.");
				String filename = new String(pData, 0, pDataLength, "UTF-8");
				//TODO: implement this.
				}
				break;
			case 0x99:
				Log.i(TAG, "Quit.");
				break;
			default:
				Log.e(TAG, String.format("Unknown command id '%s'.", pCommandID));
			}
			return result;
		}
	}
	
	private String GetMacInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "mac");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String mac = bufferedreader.readLine();
			bufferedreader.close();
			if (mac == null) {
				mac = "";
			}
			return mac;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private String GetTerminalNameInfo() {
		try {
			UserPath userpath = new UserPath();
			FileReader filereader = new FileReader(userpath.GetUserPath() + "terminalname");
			BufferedReader bufferedreader = new BufferedReader(filereader);
			String terminalname = bufferedreader.readLine();
			bufferedreader.close();
			if (terminalname == null) {
				terminalname = "";
			}
			return terminalname;
		}
		catch (Exception e) {
			return "";
		}
	}
	
	private class ServerConnectionThread extends Thread {
		private Context mContext;
		private ServerSocket mServerSocket;
		
		public ServerConnectionThread(Context pContext) {
			mContext = pContext;
		}
		
		public void Cancel() {
			try {
				mServerSocket.close();
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when close server socket: '%s - %s'.", e.getClass().getName(), e.getMessage()));
			}
		}
		
		public void run() {
			try {
				mServerSocket = new ServerSocket();
				mServerSocket.setReuseAddress(true);
				mServerSocket.bind(new InetSocketAddress(9500));
				while (! interrupted()) {
					Socket socket = null;
					try {
						socket = mServerSocket.accept();
					}
					catch (Exception e) {
						Log.e(TAG, String.format("Error when accept client '%s - %s'.", e.getClass().getName(), e.getMessage()));
						continue;
					}
					Thread thread = new Thread(new ClientConnection(mContext, socket));
					thread.start();
				}
			}
			catch (Exception e) {
				Log.e(TAG, String.format("Error when prepare network control: '%s - %s'.", e.getClass().getName(), e.getMessage()));
			}
			finally {
				if (mServerSocket != null) {
					try {
						mServerSocket.close();
					}
					catch (Exception e) {
						Log.e(TAG, String.format("Error when close server '%s - %s'.", e.getClass().getName(), e.getMessage()));
					}
				}
			}
		}
	}
	
	public NetworkHandler(Context pContext) {
		mServerConnectionThread = new ServerConnectionThread(pContext);
	}
	
	public void Start() {
		mServerConnectionThread.start();
	}
	
	public void Stop() {
		mServerConnectionThread.interrupt();
		mServerConnectionThread.Cancel();
	}
}